@@ -11,20 +11,20 @@
    This file is part of the GNU Objective C User interface library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-   
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; see the file COPYING.LIB.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 
   <chapter>
     <heading>The theme management system</heading>
@@ -135,6 +135,7 @@
 
 #include <Foundation/NSObject.h>
 #include <Foundation/NSGeometry.h>
+#include "AppKit/NSCell.h"
 // For gradient types
 #include "AppKit/NSButtonCell.h"
 
@@ -381,7 +382,7 @@ APPKIT_EXPORT	NSString	*GSThemeDidDeactivateNotification;
  * cell and view.
  */
 - (void) drawButton: (NSRect)frame
-	         in: (NSButtonCell*)cell
+	         in: (NSCell*)cell
 	       view: (NSView*)view
 	      style: (int)style
 	      state: (GSThemeControlState)state;
