@@ -11,19 +11,19 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor,
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
    Boston, MA 02110-1301, USA.
 */ 
 
@@ -36,14 +36,15 @@
 
 @interface NSSplitView : NSView
 {
-  id	   _delegate;
-  float    _dividerWidth;
-  float    _draggedBarWidth;
-  BOOL     _isVertical;
-  NSImage *_dimpleImage;
-  NSColor *_backgroundColor; 
-  NSColor *_dividerColor;
-  BOOL     _never_displayed_before;
+  id	    _delegate;
+  float     _dividerWidth;
+  float     _draggedBarWidth;
+  BOOL      _isVertical;
+  NSImage  *_dimpleImage;
+  NSColor  *_backgroundColor; 
+  NSColor  *_dividerColor;
+  BOOL      _never_displayed_before;
+  NSString *_autosaveName;
 }
 
 - (void) setDelegate: (id)anObject;
@@ -60,6 +61,8 @@
 - (BOOL) isSubviewCollapsed: (NSView *)subview;
 - (BOOL) isPaneSplitter;
 - (void) setIsPaneSplitter: (BOOL)flag;
+- (void) setAutosaveName: (NSString *)autosaveName;
+- (NSString *) autosaveName;
 #endif
 
 @end
