@@ -9,19 +9,20 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */ 
 
 #include "config.h"
@@ -857,7 +858,12 @@ static NSString         *disabledName = @".GNUstepDisabled";
     }
   if (changed)
     {
-      [self rebuildServices];
+      /* If we have changed the enabled/disabled services,
+       * or there have been services added/removed
+       * then we must rebuild the services menu to add/remove
+       * items as appropriate.
+       */
+      [self rebuildServicesMenu];
     }
 }
 
@@ -880,6 +886,10 @@ static NSString         *disabledName = @".GNUstepDisabled";
   return _port;
 }
 
+/**
+ * Makes the current set of usable services consistent with the
+ * data types currently available.
+ */
 - (void) rebuildServices
 {
   NSDictionary          *services;
@@ -954,6 +964,9 @@ static NSString         *disabledName = @".GNUstepDisabled";
     }
 }
 
+/** Adds or removes items in the services menu in response to a change
+ * in the services which are available to the app.
+ */
 - (void) rebuildServicesMenu
 {
   if (_servicesMenu != nil)
@@ -966,9 +979,10 @@ static NSString         *disabledName = @".GNUstepDisabled";
       NSMenu            *submenu = nil;
 
       [_servicesMenu setAutoenablesItems: NO];
-      pos = [_servicesMenu numberOfItems];
-      for (; pos; pos--)
-        [_servicesMenu removeItemAtIndex: 0];
+      for (pos = [_servicesMenu numberOfItems]; pos > 0; pos--)
+        {
+          [_servicesMenu removeItemAtIndex: 0];
+        }
       [_servicesMenu setAutoenablesItems: YES];
 
       keyEquivalents = [NSMutableSet setWithCapacity: 4];
@@ -976,17 +990,23 @@ static NSString         *disabledName = @".GNUstepDisabled";
         {
           NSString      *title = [_menuTitles objectAtIndex: pos];
           NSString      *equiv = @"";
-          NSDictionary  *info = [_title2info objectForKey: title];
+          NSDictionary  *info;
           NSDictionary  *titles;
           NSDictionary  *equivs;
           NSRange       r;
           unsigned      lang;
           id<NSMenuItem>        item;
 
+          if (NSShowsServicesMenuItem(title) == NO)
+            {
+              continue; // We don't want to show this one.
+            }
+
           /*
            *    Find the key equivalent corresponding to this menu title
            *    in the service definition.
            */
+          info = [_title2info objectForKey: title];
           titles = [info objectForKey: @"NSMenuItem"];
           equivs = [info objectForKey: @"NSKeyEquivalent"];
           for (lang = 0; lang < [_languages count]; lang++)
@@ -1215,6 +1235,9 @@ static NSString         *disabledName = @".GNUstepDisabled";
 
   if (didChange)
     {
+      /* Types have changed, so we need to enable/disable items in the
+       * services menu depending on what types they support.
+       */
       [self rebuildServices];
     }
 }
