@@ -10,20 +10,21 @@
    This file is part of the GNU Objective C User interface library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-   
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
-   */
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; see the file COPYING.LIB.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
+*/
 
 #include "Foundation/NSBundle.h"
 #include "Foundation/NSDictionary.h"
@@ -653,7 +654,8 @@ static NSNull			*null = nil;
 	  unsigned	count;
 
 	  imageTypes = [NSImage imageFileTypes];
-	  for (count = 0; image == nil && count < [imageTypes count]; count++)
+	  for (count = 0, image = nil; image == nil && count < [imageTypes count]; 
+         count++)
 	    {
 	      NSString	*ext = [imageTypes objectAtIndex: count];
 
@@ -696,7 +698,7 @@ static NSNull			*null = nil;
 @implementation	GSTheme (Drawing)
 
 - (void) drawButton: (NSRect)frame 
-                 in: (NSButtonCell*)cell 
+                 in: (NSCell*)cell 
                view: (NSView*)view 
               style: (int)style 
               state: (GSThemeControlState)state
