@@ -8,35 +8,37 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */
 
-#include <AppKit/NSView.h>
-#include <AppKit/NSWindow.h>
-#include <AppKit/NSPanel.h>
-#include <AppKit/NSButton.h>
-#include <AppKit/NSImage.h>
-#include <AppKit/NSMenu.h>
-#include <AppKit/NSMenuView.h>
-#include <AppKit/NSApplication.h>
-
-#include <GNUstepGUI/GSTitleView.h>
-
 #include <Foundation/NSDebug.h>
 #include <Foundation/NSRunLoop.h>
 
+#include <AppKit/NSApplication.h>
+#include "AppKit/NSAttributedString.h"
+#include <AppKit/NSButton.h>
+#include <AppKit/NSEvent.h>
+#include <AppKit/NSImage.h>
+#include <AppKit/NSMenu.h>
+#include <AppKit/NSMenuView.h>
+#include <AppKit/NSPanel.h>
+#include <AppKit/NSView.h>
+#include <AppKit/NSWindow.h>
+
+#include <GNUstepGUI/GSTitleView.h>
 #include "GNUstepGUI/GSTheme.h"
 
 @implementation GSTitleView
