@@ -11,20 +11,21 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-   
-   You should have received a copy of the GNU Library General Public
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-   */
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
+*/
 
 #include "config.h"
 #include "NSBitmapImageRep+PNG.h"
@@ -209,14 +210,15 @@ static void reader_func(png_structp png_struct, png_bytep data,
   }
 
   [self initWithBitmapDataPlanes: &buf
-		      pixelsWide: width
-		      pixelsHigh: height
-		   bitsPerSample: depth
-		 samplesPerPixel: channels
-			hasAlpha: alpha
-			isPlanar: NO
-		  colorSpaceName: colorspace
-		     bytesPerRow: bytes_per_row
+        pixelsWide: width
+        pixelsHigh: height
+        bitsPerSample: depth
+        samplesPerPixel: channels
+        hasAlpha: alpha
+        isPlanar: NO
+        colorSpaceName: colorspace
+        bitmapFormat: NSAlphaNonpremultipliedBitmapFormat
+        bytesPerRow: bytes_per_row
 		    bitsPerPixel: bpp];
 
   _imageData = [[NSData alloc]
@@ -273,6 +275,7 @@ static void writer_func(png_structp png_struct, png_bytep data,
   NSNumber * gammaNumber = nil;
   double gamma = 0.0;
   
+  // FIXME: Need to convert to non-pre-multiplied format
   if ([self isPlanar])	// don't handle planar yet
   {
     return nil;
