@@ -14,19 +14,20 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
-   License along with this library;
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; see the file COPYING.LIB.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */ 
 
 #include "config.h"
@@ -185,7 +186,7 @@
 @implementation	NSNibControlConnector
 - (void) establishConnection
 {
-  SEL		sel = NSSelectorFromString(_tag);
+  SEL sel = NSSelectorFromString(_tag);
 	      
   [_src setTarget: _dst];
   [_src setAction: sel];
@@ -197,8 +198,8 @@
 {
   if (_src != nil)
     {
-      NSString	*selName;
-      SEL	sel;
+      NSString *selName;
+      SEL sel;
 
       selName = [NSString stringWithFormat: @"set%@%@:",
 			  [[_tag substringToIndex: 1] uppercaseString],
@@ -211,10 +212,10 @@
 	}
       else
 	{
-	  const char	*nam = [_tag cString];
-	  const char	*type;
-	  unsigned int	size;
-	  int	offset;
+	  const char *nam = [_tag cString];
+	  const char *type;
+	  unsigned int size;
+	  int offset;
 
 	  /*
 	   * Use the GNUstep additional function to set the instance
