@@ -10,18 +10,20 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; see the file COPYING.LIB.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */
 
 #include "config.h"
@@ -30,6 +32,7 @@
 #include "AppKit/NSClipView.h"
 #include "AppKit/NSCursor.h"
 #include "AppKit/NSColor.h"
+#include "AppKit/NSEvent.h"
 #include "AppKit/NSGraphics.h"
 #include "AppKit/NSTableView.h"
 #include "AppKit/NSWindow.h"
@@ -211,6 +214,7 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
 - (void) scrollToPoint: (NSPoint)aPoint
 {
   [self setBoundsOrigin: [self constrainScrollPoint: aPoint]];
+  [self resetCursorRects];
 }
 
 - (void) setBounds: (NSRect)b
@@ -290,7 +294,7 @@ static inline NSRect integralRect (NSRect rect, NSView *view)
           NSRect redrawRect;
                     
           /* Copy the intersection to the new position */
-          [self scrollRect: intersection by: NSMakeSize(dx, dy)];
+          [self scrollRect: intersection by: NSMakeSize(-dx, -dy)];
 
           /* Change coordinate system to the new one */
           [super setBoundsOrigin: newBounds.origin];
