@@ -11,27 +11,30 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */ 
 
 #include "config.h"
 #include <Foundation/NSArray.h>
+#include <Foundation/NSDictionary.h>
 #include <Foundation/NSSet.h>
 #include <Foundation/NSString.h>
 #include <Foundation/NSValue.h>
 #include <Foundation/NSDebug.h>
+#include "AppKit/NSFontDescriptor.h"
 #include "AppKit/NSFontManager.h"
 #include "AppKit/NSApplication.h"
 #include "AppKit/NSFont.h"
@@ -44,10 +47,10 @@
 /*
  * Class variables
  */
-static NSFontManager	*sharedFontManager = nil;
-static NSFontPanel	*fontPanel = nil;
-static Class		fontManagerClass = Nil;
-static Class		fontPanelClass = Nil;
+static NSFontManager *sharedFontManager = nil;
+static NSFontPanel   *fontPanel = nil;
+static Class         fontManagerClass = Nil;
+static Class         fontPanelClass = Nil;
 
 
 @implementation NSFontManager
@@ -116,6 +119,7 @@ static Class		fontPanelClass = Nil;
   _action = @selector(changeFont:);
   _storedTag = NSNoFontChangeAction;
   _fontEnumerator = RETAIN([GSFontEnumerator sharedEnumerator]);
+  _collections = [[NSMutableDictionary alloc] initWithCapacity: 3];
 
   return self;
 }
@@ -123,8 +127,10 @@ static Class		fontPanelClass = Nil;
 - (void) dealloc
 {
   TEST_RELEASE(_selectedFont);
+  TEST_RELEASE(_selectedAttributes);
   TEST_RELEASE(_fontMenu);
   TEST_RELEASE(_fontEnumerator);
+  RELEASE(_collections);
   [super dealloc];
 }
 
@@ -150,17 +156,17 @@ static Class		fontPanelClass = Nil;
   for (i = 0; i < [fontFamilies count]; i++)
     {
       NSArray *fontDefs = [self availableMembersOfFontFamily: 
-				 [fontFamilies objectAtIndex: i]];
+                                 [fontFamilies objectAtIndex: i]];
       
       for (j = 0; j < [fontDefs count]; j++)
-	{
-	  NSArray	*fontDef = [fontDefs objectAtIndex: j];
+        {
+          NSArray        *fontDef = [fontDefs objectAtIndex: j];
 
-	  traits = [[fontDef objectAtIndex: 3] unsignedIntValue];
-	  // Check if the font has exactly the given mask
-	  if (traits == fontTraitMask)
-	    [fontNames addObject: [fontDef objectAtIndex: 0]];
-	}
+          traits = [[fontDef objectAtIndex: 3] unsignedIntValue];
+          // Check if the font has exactly the given mask
+          if (traits == fontTraitMask)
+            [fontNames addObject: [fontDef objectAtIndex: 0]];
+        }
     }
 
   return fontNames;
@@ -172,7 +178,7 @@ static Class		fontPanelClass = Nil;
 }
 
 - (NSString*) localizedNameForFamily: (NSString*)family 
-				face: (NSString*)face
+                                face: (NSString*)face
 {
   // TODO
   return [NSString stringWithFormat: @"%@-%@", family, face];
@@ -181,24 +187,25 @@ static Class		fontPanelClass = Nil;
 /**
  */
 - (void) setSelectedFont: (NSFont*)fontObject
-	      isMultiple: (BOOL)flag
+              isMultiple: (BOOL)flag
 {
   if (_selectedFont == fontObject)
     {
       if (flag != _multiple)
-	{
-	  _multiple = flag;
-	  // The panel should also know if multiple changed
-	  if (fontPanel != nil)
-	    {
-	      [fontPanel setPanelFont: fontObject isMultiple: flag];
-	    }
-	}
+        {
+          _multiple = flag;
+          // The panel should also know if multiple changed
+          if (fontPanel != nil)
+            {
+              [fontPanel setPanelFont: fontObject isMultiple: flag];
+            }
+        }
       return;
     }
 
   _multiple = flag;
   ASSIGN(_selectedFont, fontObject);
+  DESTROY(_selectedAttributes);
 
   if (fontPanel != nil)
     {
@@ -215,42 +222,42 @@ static Class		fontPanelClass = Nil;
        * We keep the tag, to mark the item
        */
       if (trait & NSItalicFontMask)
-	{
-	  menuItem = [_fontMenu itemWithTag: NSItalicFontMask];
-	  if (menuItem != nil)
-	    {
-	      [menuItem setTitle: @"Unitalic"];
-	      [menuItem setAction: @selector(removeFontTrait:)];
-	    }
-	}
+        {
+          menuItem = [_fontMenu itemWithTag: NSItalicFontMask];
+          if (menuItem != nil)
+            {
+              [menuItem setTitle: @"Unitalic"];
+              [menuItem setAction: @selector(removeFontTrait:)];
+            }
+        }
       else
-	{
-	  menuItem = [_fontMenu itemWithTag: NSItalicFontMask];
-	  if (menuItem != nil)
-	    {
-	      [menuItem setTitle: @"Italic"];
-	      [menuItem setAction: @selector(addFontTrait:)];
-	    }
-	}
+        {
+          menuItem = [_fontMenu itemWithTag: NSItalicFontMask];
+          if (menuItem != nil)
+            {
+              [menuItem setTitle: @"Italic"];
+              [menuItem setAction: @selector(addFontTrait:)];
+            }
+        }
 
       if (trait & NSBoldFontMask)
-	{
-	  menuItem = [_fontMenu itemWithTag: NSBoldFontMask];
-	  if (menuItem != nil)
-	    {
-	      [menuItem setTitle: @"Unbold"];
-	      [menuItem setAction: @selector(removeFontTrait:)];
-	    }
-	}
+        {
+          menuItem = [_fontMenu itemWithTag: NSBoldFontMask];
+          if (menuItem != nil)
+            {
+              [menuItem setTitle: @"Unbold"];
+              [menuItem setAction: @selector(removeFontTrait:)];
+            }
+        }
       else
-	{
-	  menuItem = [_fontMenu itemWithTag: NSBoldFontMask];
-	  if (menuItem != nil)
-	    {
-	      [menuItem setTitle: @"Bold"];
-	      [menuItem setAction: @selector(addFontTrait:)];
-	    }
-	}
+        {
+          menuItem = [_fontMenu itemWithTag: NSBoldFontMask];
+          if (menuItem != nil)
+            {
+              [menuItem setTitle: @"Bold"];
+              [menuItem setAction: @selector(addFontTrait:)];
+            }
+        }
 
       // TODO Update the rest of the font menu to reflect this font
     }
@@ -283,12 +290,12 @@ static Class		fontPanelClass = Nil;
   // We update our own selected font
   if (_selectedFont != nil)
     {
-      NSFont	*newFont = [self convertFont: _selectedFont];
+      NSFont *newFont = [self convertFont: _selectedFont];
 
       if (newFont != nil)
-	{
-	  [self setSelectedFont: newFont isMultiple: _multiple];
-	}
+        {
+          [self setSelectedFont: newFont isMultiple: _multiple];
+        }
     }
 }
 
@@ -301,12 +308,12 @@ static Class		fontPanelClass = Nil;
   // We update our own selected font
   if (_selectedFont != nil)
     {
-      NSFont	*newFont = [self convertFont: _selectedFont];
+      NSFont *newFont = [self convertFont: _selectedFont];
 
       if (newFont != nil)
-	{
-	  [self setSelectedFont: newFont isMultiple: _multiple];
-	}
+        {
+          [self setSelectedFont: newFont isMultiple: _multiple];
+        }
     }
 }
 
@@ -318,12 +325,12 @@ static Class		fontPanelClass = Nil;
   // We update our own selected font
   if (_selectedFont != nil)
     {
-      NSFont	*newFont = [self convertFont: _selectedFont];
+      NSFont *newFont = [self convertFont: _selectedFont];
 
       if (newFont != nil)
-	{
-	  [self setSelectedFont: newFont isMultiple: _multiple];
-	}
+        {
+          [self setSelectedFont: newFont isMultiple: _multiple];
+        }
     }
 }
 
@@ -335,12 +342,12 @@ static Class		fontPanelClass = Nil;
   // We update our own selected font
   if (_selectedFont != nil)
     {
-      NSFont	*newFont = [self convertFont: _selectedFont];
+      NSFont *newFont = [self convertFont: _selectedFont];
 
       if (newFont != nil)
-	{
-	  [self setSelectedFont: newFont isMultiple: _multiple];
-	}
+        {
+          [self setSelectedFont: newFont isMultiple: _multiple];
+        }
     }
 }
 
@@ -356,7 +363,7 @@ static Class		fontPanelClass = Nil;
   unsigned int i;
   float size;
   float sizes[] = {4.0, 6.0, 8.0, 9.0, 10.0, 11.0, 12.0, 13.0, 
-		   14.0, 16.0, 18.0, 24.0, 36.0, 48.0, 64.0};
+                   14.0, 16.0, 18.0, 24.0, 36.0, 48.0, 64.0};
 
   if (fontObject == nil)
     return nil;
@@ -364,51 +371,51 @@ static Class		fontPanelClass = Nil;
   switch (_storedTag)
     {
       case NSNoFontChangeAction: 
-	break;
+        break;
       case NSViaPanelFontAction: 
-	if (fontPanel != nil)
-	  {
-	    newFont = [fontPanel panelConvertFont: fontObject];
-	  }
-	break;
+        if (fontPanel != nil)
+          {
+            newFont = [fontPanel panelConvertFont: fontObject];
+          }
+        break;
       case NSAddTraitFontAction: 
-	newFont = [self convertFont: fontObject toHaveTrait: _trait];
-	break;
+        newFont = [self convertFont: fontObject toHaveTrait: _trait];
+        break;
       case NSRemoveTraitFontAction: 
-	 newFont = [self convertFont: fontObject toNotHaveTrait: _trait];
-	break;
+         newFont = [self convertFont: fontObject toNotHaveTrait: _trait];
+        break;
       case NSSizeUpFontAction: 
-	size = [fontObject pointSize];
-	for (i = 0; i < sizeof(sizes)/sizeof(float); i++)
-	  {
-	    if (sizes[i] > size)
-	      {
-		size = sizes[i];
-		break;
-	      }
-	  }
-	newFont = [self convertFont: fontObject 
-			toSize: size];
-	break;
+        size = [fontObject pointSize];
+        for (i = 0; i < sizeof(sizes)/sizeof(float); i++)
+          {
+            if (sizes[i] > size)
+              {
+                size = sizes[i];
+                break;
+              }
+          }
+        newFont = [self convertFont: fontObject 
+                        toSize: size];
+        break;
       case NSSizeDownFontAction: 
-	size = [fontObject pointSize];
-	for (i = sizeof(sizes)/sizeof(float) -1; i >= 0; i--)
-	  {
-	    if (sizes[i] < size)
-	      {
-		size = sizes[i];
-		break;
-	      }
-	  }
-	newFont = [self convertFont: fontObject 
-			toSize: size];
-	break;
+        size = [fontObject pointSize];
+        for (i = sizeof(sizes)/sizeof(float) -1; i >= 0; i--)
+          {
+            if (sizes[i] < size)
+              {
+                size = sizes[i];
+                break;
+              }
+          }
+        newFont = [self convertFont: fontObject 
+                        toSize: size];
+        break;
       case NSHeavierFontAction: 
-	newFont = [self convertWeight: YES ofFont: fontObject]; 
-	break;
+        newFont = [self convertWeight: YES ofFont: fontObject]; 
+        break;
       case NSLighterFontAction: 
-	newFont = [self convertWeight: NO ofFont: fontObject]; 
-	break;
+        newFont = [self convertWeight: NO ofFont: fontObject]; 
+        break;
     }
 
   return newFont;
@@ -416,7 +423,7 @@ static Class		fontPanelClass = Nil;
 
 
 - (NSFont*) convertFont: (NSFont*)fontObject
-	       toFamily: (NSString*)family
+               toFamily: (NSString*)family
 {
   if ([family isEqualToString: [fontObject familyName]])
     {
@@ -432,18 +439,18 @@ static Class		fontPanelClass = Nil;
       float size = [fontObject pointSize];
 
       newFont = [self fontWithFamily: family 
-			      traits: trait
-			      weight: weight
-				size: size];
+                              traits: trait
+                              weight: weight
+                                size: size];
       if (newFont == nil)
-	return fontObject;
+        return fontObject;
       else 
-	return newFont;
+        return newFont;
     }
 }
 
 - (NSFont*) convertFont: (NSFont*)fontObject
-		 toFace: (NSString*)typeface
+                 toFace: (NSString*)typeface
 {
   NSFont *newFont;
 
@@ -461,7 +468,7 @@ static Class		fontPanelClass = Nil;
 }
 
 - (NSFont*) convertFont: (NSFont*)fontObject
-	    toHaveTrait: (NSFontTraitMask)trait
+            toHaveTrait: (NSFontTraitMask)trait
 {
   NSFontTraitMask t = [self traitsOfFont: fontObject];
 
@@ -473,13 +480,13 @@ static Class		fontPanelClass = Nil;
   else if (trait == NSUnboldFontMask)
     {
       return [self convertFont: fontObject 
-		toNotHaveTrait: NSBoldFontMask];
-   }
+                   toNotHaveTrait: NSBoldFontMask];
+    }
   else if (trait == NSUnitalicFontMask)
     {
       return [self convertFont: fontObject 
-		toNotHaveTrait: NSItalicFontMask];
-   }
+                   toNotHaveTrait: NSItalicFontMask];
+    }
   else
     {
       // Else convert it
@@ -489,37 +496,36 @@ static Class		fontPanelClass = Nil;
       float size = [fontObject pointSize];
       NSString *family = [fontObject familyName];
 
-      // We cannot reuse the weight in a bold
-      if (trait == NSBoldFontMask)
-	weight = 9;
+      if (trait & NSBoldFontMask)
+        {
+          // We cannot reuse the weight in a bold
+          weight = 9;
+          t = t & ~NSUnboldFontMask;
+        }
+      if (trait == NSItalicFontMask)
+        {
+          t = t & ~NSUnitalicFontMask;
+        }
 
       t = t | trait;
+
       newFont = [self fontWithFamily: family 
-			      traits: t
-			      weight: weight
-				size: size];
+                              traits: t
+                              weight: weight
+                                size: size];
 
       if (newFont == nil)
-	return fontObject;
+        return fontObject;
       else 
-	return newFont;
+        return newFont;
     }
 }
 
 - (NSFont*) convertFont: (NSFont*)fontObject
-	 toNotHaveTrait: (NSFontTraitMask)trait
+         toNotHaveTrait: (NSFontTraitMask)trait
 {
   NSFontTraitMask t = [self traitsOfFont: fontObject];
 
-  // This is a bit strange but is stated in the specification
-  if (trait & NSUnboldFontMask)
-    {
-      trait = (trait | NSBoldFontMask) & ~NSUnboldFontMask;
-    }
-  if (trait & NSUnitalicFontMask)
-    {
-      trait = (trait | NSItalicFontMask) & ~NSUnitalicFontMask;
-    }
   if (!(t & trait))
     {
       // If already do not have that trait then just return it
@@ -534,25 +540,41 @@ static Class		fontPanelClass = Nil;
       float size = [fontObject pointSize];
       NSString *family = [fontObject familyName];
 
-      // We cannot reuse the weight in an unbold
       if (trait & NSBoldFontMask)
-	{
-	  weight = 5;
-	}
+        {
+          // We cannot reuse the weight in an unbold
+          weight = 5;
+          t = (t | NSUnboldFontMask);
+        }
+      else if (trait & NSUnboldFontMask)
+        {
+          // We cannot reuse the weight in a bold
+          weight = 9;
+          t = (t | NSBoldFontMask);
+        }
+      if (trait & NSItalicFontMask)
+        {
+          t = (t | NSUnitalicFontMask);
+        }
+      else if (trait & NSUnitalicFontMask)
+        {
+          t = (t | NSItalicFontMask);
+        }
+ 
       t &= ~trait;
       newFont = [self fontWithFamily: family 
-			      traits: t
-			      weight: weight
-				size: size];
+                              traits: t
+                              weight: weight
+                                size: size];
       if (newFont == nil)
-	return fontObject;
+        return fontObject;
       else 
-	return newFont;
+        return newFont;
     }
 }
 
 - (NSFont*) convertFont: (NSFont*)fontObject
-		 toSize: (float)size
+                 toSize: (float)size
 {
   if ([fontObject pointSize] == size)
     {
@@ -565,16 +587,16 @@ static Class		fontPanelClass = Nil;
       NSFont *newFont;
 
       newFont = [NSFont fontWithName: [fontObject fontName] 
-			size: size];
+                        size: size];
       if (newFont == nil)
-	return fontObject;
+        return fontObject;
       else 
-	return newFont;
+        return newFont;
     }
 }
 
 - (NSFont*) convertWeight: (BOOL)upFlag
-		   ofFont: (NSFont*)fontObject
+                   ofFont: (NSFont*)fontObject
 {
   NSFont *newFont = nil;
   NSString *fontName = nil;
@@ -594,36 +616,36 @@ static Class		fontPanelClass = Nil;
       int next_w = 15;
 
       for (i = 0; i < [fontDefs count]; i++)
-	{
-	  NSArray *fontDef = [fontDefs objectAtIndex: i];
-	  int w1 = [[fontDef objectAtIndex: 2] intValue];
+        {
+          NSArray *fontDef = [fontDefs objectAtIndex: i];
+          int w1 = [[fontDef objectAtIndex: 2] intValue];
 
-	  if (w1 > w && w1 < next_w && 
-	      [[fontDef objectAtIndex: 3] unsignedIntValue] == trait)
-	    {
-	      next_w = w1;
-	      fontName = [fontDef objectAtIndex: 0];
-	    }
-	}
+          if (w1 > w && w1 < next_w && 
+              [[fontDef objectAtIndex: 3] unsignedIntValue] == trait)
+            {
+              next_w = w1;
+              fontName = [fontDef objectAtIndex: 0];
+            }
+        }
 
       if (fontName == nil)
         {
-	  // Not found, try again with changed trait
-	  trait |= NSBoldFontMask;
-	  
-	  for (i = 0; i < [fontDefs count]; i++)
-	    { 
-	      NSArray *fontDef = [fontDefs objectAtIndex: i];
-	      int w1 = [[fontDef objectAtIndex: 2] intValue];
+          // Not found, try again with changed trait
+          trait |= NSBoldFontMask;
+          
+          for (i = 0; i < [fontDefs count]; i++)
+            { 
+              NSArray *fontDef = [fontDefs objectAtIndex: i];
+              int w1 = [[fontDef objectAtIndex: 2] intValue];
 
-	      if (w1 > w && w1 < next_w && 
-		  [[fontDef objectAtIndex: 3] unsignedIntValue] == trait)
-	        {
-		  next_w = w1;
-		  fontName = [fontDef objectAtIndex: 0];
-		}
-	    }
-	}
+              if (w1 > w && w1 < next_w && 
+                  [[fontDef objectAtIndex: 3] unsignedIntValue] == trait)
+                {
+                  next_w = w1;
+                  fontName = [fontDef objectAtIndex: 0];
+                }
+            }
+        }
     }
   else
     {
@@ -631,42 +653,42 @@ static Class		fontPanelClass = Nil;
       int next_w = 0;
 
       for (i = 0; i < [fontDefs count]; i++)
-	{
-	  NSArray *fontDef = [fontDefs objectAtIndex: i];
-	  int w1 = [[fontDef objectAtIndex: 2] intValue];
+        {
+          NSArray *fontDef = [fontDefs objectAtIndex: i];
+          int w1 = [[fontDef objectAtIndex: 2] intValue];
 
-	  if (w1 < w && w1 > next_w
-	    && [[fontDef objectAtIndex: 3] unsignedIntValue] == trait)
-	    {
-	      next_w = w1;
-	      fontName = [fontDef objectAtIndex: 0];
-	    }
-	}
+          if (w1 < w && w1 > next_w
+            && [[fontDef objectAtIndex: 3] unsignedIntValue] == trait)
+            {
+              next_w = w1;
+              fontName = [fontDef objectAtIndex: 0];
+            }
+        }
 
       if (fontName == nil)
         {
-	  // Not found, try again with changed trait
-	  trait &= ~NSBoldFontMask;
+          // Not found, try again with changed trait
+          trait &= ~NSBoldFontMask;
 
-	  for (i = 0; i < [fontDefs count]; i++)
-	    {
-	      NSArray *fontDef = [fontDefs objectAtIndex: i];
-	      int w1 = [[fontDef objectAtIndex: 2] intValue];
-	      
-	      if (w1 < w && w1 > next_w
-		&& [[fontDef objectAtIndex: 3] unsignedIntValue] == trait)
-	        {
-		  next_w = w1;
-		  fontName = [fontDef objectAtIndex: 0];
-		}
-	    }
-	}
+          for (i = 0; i < [fontDefs count]; i++)
+            {
+              NSArray *fontDef = [fontDefs objectAtIndex: i];
+              int w1 = [[fontDef objectAtIndex: 2] intValue];
+              
+              if (w1 < w && w1 > next_w
+                && [[fontDef objectAtIndex: 3] unsignedIntValue] == trait)
+                {
+                  next_w = w1;
+                  fontName = [fontDef objectAtIndex: 0];
+                }
+            }
+        }
     }
 
   if (fontName != nil)
     {
       newFont = [NSFont fontWithName: fontName
-				size: size];
+                                size: size];
     }
   if (newFont == nil)
     return fontObject;
@@ -678,9 +700,9 @@ static Class		fontPanelClass = Nil;
  * Getting a font
  */
 - (NSFont*) fontWithFamily: (NSString*)family
-		    traits: (NSFontTraitMask)traits
-		    weight: (int)weight
-		      size: (float)size
+                    traits: (NSFontTraitMask)traits
+                    weight: (int)weight
+                      size: (float)size
 {
   NSArray *fontDefs = [self availableMembersOfFontFamily: family];
   unsigned int i;
@@ -696,34 +718,50 @@ static Class		fontPanelClass = Nil;
       //          [[fontDef objectAtIndex: 2] intValue], 
       //          [[fontDef objectAtIndex: 3] unsignedIntValue]);  
       if (([[fontDef objectAtIndex: 2] intValue] == weight) &&
-	  ([[fontDef objectAtIndex: 3] unsignedIntValue] == traits))
-	{
-          //NSLog(@"Found font");
-	  return [NSFont fontWithName: [fontDef objectAtIndex: 0] 
-			 size: size];
-	}
+          ([[fontDef objectAtIndex: 3] unsignedIntValue] == traits))
+        {
+            //NSLog(@"Found font");
+          return [NSFont fontWithName: [fontDef objectAtIndex: 0] 
+                         size: size];
+        }
     }
 
-  // Try to find something close
+  // Try to find something close by ignoring some trait flags
+  traits &= ~(NSNonStandardCharacterSetFontMask | NSFixedPitchFontMask
+              | NSUnitalicFontMask | NSUnboldFontMask);
+  for (i = 0; i < [fontDefs count]; i++)
+    {
+      NSArray *fontDef = [fontDefs objectAtIndex: i];
+      NSFontTraitMask t = [[fontDef objectAtIndex: 3] unsignedIntValue];
 
-  traits &= ~(NSNonStandardCharacterSetFontMask | NSFixedPitchFontMask);
+      t &= ~(NSNonStandardCharacterSetFontMask | NSFixedPitchFontMask
+             | NSUnitalicFontMask | NSUnboldFontMask);
+      if (([[fontDef objectAtIndex: 2] intValue] == weight) &&
+          (t == traits))
+        {
+            //NSLog(@"Found font");
+          return [NSFont fontWithName: [fontDef objectAtIndex: 0] 
+                         size: size];
+        }
+    }
 
   if (traits & NSBoldFontMask)
     {
       //NSLog(@"Trying ignore weights for bold font");
       for (i = 0; i < [fontDefs count]; i++)
         {
-	  NSArray *fontDef = [fontDefs objectAtIndex: i];
-	  NSFontTraitMask t = [[fontDef objectAtIndex: 3] unsignedIntValue];
+          NSArray *fontDef = [fontDefs objectAtIndex: i];
+          NSFontTraitMask t = [[fontDef objectAtIndex: 3] unsignedIntValue];
 
-	  t &= ~(NSNonStandardCharacterSetFontMask | NSFixedPitchFontMask);
-	  if (t == traits)
-	    {
-	      //NSLog(@"Found font");
-	      return [NSFont fontWithName: [fontDef objectAtIndex: 0] 
-			     size: size];
-	    }
-	}
+          t &= ~(NSNonStandardCharacterSetFontMask | NSFixedPitchFontMask
+                 | NSUnitalicFontMask | NSUnboldFontMask);
+          if (t == traits)
+            {
+              //NSLog(@"Found font");
+              return [NSFont fontWithName: [fontDef objectAtIndex: 0] 
+                             size: size];
+            }
+        }
     }
   
   if (weight == 5 || weight == 6)
@@ -731,19 +769,20 @@ static Class		fontPanelClass = Nil;
       //NSLog(@"Trying alternate non-bold weights for non-bold font");
       for (i = 0; i < [fontDefs count]; i++)
         {
-	  NSArray *fontDef = [fontDefs objectAtIndex: i];
-	  NSFontTraitMask t = [[fontDef objectAtIndex: 3] unsignedIntValue];
+          NSArray *fontDef = [fontDefs objectAtIndex: i];
+          NSFontTraitMask t = [[fontDef objectAtIndex: 3] unsignedIntValue];
 
-	  t &= ~(NSNonStandardCharacterSetFontMask | NSFixedPitchFontMask);
-	  if ((([[fontDef objectAtIndex: 2] intValue] == 5) ||
+          t &= ~(NSNonStandardCharacterSetFontMask | NSFixedPitchFontMask
+                 | NSUnitalicFontMask | NSUnboldFontMask);
+          if ((([[fontDef objectAtIndex: 2] intValue] == 5) ||
                ([[fontDef objectAtIndex: 2] intValue] == 6)) &&
-	      (t == traits))
-	    {
-	      //NSLog(@"Found font");
-	      return [NSFont fontWithName: [fontDef objectAtIndex: 0] 
-			     size: size];
-	    }
-	}
+              (t == traits))
+            {
+              //NSLog(@"Found font");
+              return [NSFont fontWithName: [fontDef objectAtIndex: 0] 
+                             size: size];
+            }
+        }
     }
 
   //NSLog(@"Didnt find font");  
@@ -777,24 +816,24 @@ static Class		fontPanelClass = Nil;
   for (i = 0; i < [fontFamilies count]; i++)
     {
       NSArray *fontDefs = [self availableMembersOfFontFamily: 
-				  [fontFamilies objectAtIndex: i]];
+                                  [fontFamilies objectAtIndex: i]];
       
       for (j = 0; j < [fontDefs count]; j++)
-	{
-	  NSArray *fontDef = [fontDefs objectAtIndex: j];
-	  
-	  if ([[fontDef objectAtIndex: 0] isEqualToString: typeface])
-	    {
-	      traits = [[fontDef objectAtIndex: 3] unsignedIntValue];
-	      // FIXME: This is not exactly the right condition
-	      if ((traits & fontTraitMask) == fontTraitMask)
-		{
-		  return YES;
-		}
-	      else
-		return NO;
-	    }
-	}
+        {
+          NSArray *fontDef = [fontDefs objectAtIndex: j];
+          
+          if ([[fontDef objectAtIndex: 0] isEqualToString: typeface])
+            {
+              traits = [[fontDef objectAtIndex: 3] unsignedIntValue];
+              // FIXME: This is not exactly the right condition
+              if ((traits & fontTraitMask) == fontTraitMask)
+                {
+                  return YES;
+                }
+              else
+                return NO;
+            }
+        }
     }
   
   return NO;
@@ -822,9 +861,9 @@ static Class		fontPanelClass = Nil;
   if (_fontMenu != nil)
     {
       for (i = 0; i < [_fontMenu numberOfItems]; i++)
-	{
-	  [[_fontMenu itemAtIndex: i] setEnabled: flag];
-	}
+        {
+          [[_fontMenu itemAtIndex: i] setEnabled: flag];
+        }
     }
 
   if (fontPanel != nil)
@@ -847,49 +886,49 @@ static Class		fontPanelClass = Nil;
 
       // First an entry to start the font panel
       menuItem = [_fontMenu addItemWithTitle: @"Font Panel"
-			    action: @selector(orderFrontFontPanel:)
-			    keyEquivalent: @"t"];
+                            action: @selector(orderFrontFontPanel:)
+                            keyEquivalent: @"t"];
       [menuItem setTarget: self];
 
       // Entry for italic
       menuItem = [_fontMenu addItemWithTitle: @"Italic"
-			    action: @selector(addFontTrait:)
-			    keyEquivalent: @"i"];
+                            action: @selector(addFontTrait:)
+                            keyEquivalent: @"i"];
       [menuItem setTag: NSItalicFontMask];
       [menuItem setTarget: self];
 
       // Entry for bold
       menuItem = [_fontMenu addItemWithTitle: @"Bold"
-			    action: @selector(addFontTrait:)
-			    keyEquivalent: @"b"];
+                            action: @selector(addFontTrait:)
+                            keyEquivalent: @"b"];
       [menuItem setTag: NSBoldFontMask];
       [menuItem setTarget: self];
 
       // Entry to increase weight
       menuItem = [_fontMenu addItemWithTitle: @"Heavier"
-			    action: @selector(modifyFont:)
-			    keyEquivalent: @"h"];
+                            action: @selector(modifyFont:)
+                            keyEquivalent: @"h"];
       [menuItem setTag: NSHeavierFontAction];
       [menuItem setTarget: self];
  
       // Entry to decrease weight
       menuItem = [_fontMenu addItemWithTitle: @"Lighter"
-			    action: @selector(modifyFont:)
-			    keyEquivalent: @"g"];
+                            action: @selector(modifyFont:)
+                            keyEquivalent: @"g"];
       [menuItem setTag: NSLighterFontAction];
       [menuItem setTarget: self];
  
       // Entry to increase size
       menuItem = [_fontMenu addItemWithTitle: @"Larger"
-			    action: @selector(modifyFont:)
-			    keyEquivalent: @"l"];
+                            action: @selector(modifyFont:)
+                            keyEquivalent: @"l"];
       [menuItem setTag: NSSizeUpFontAction];
       [menuItem setTarget: self];
 
       // Entry to decrease size
       menuItem = [_fontMenu addItemWithTitle: @"Smaller"
-			    action: @selector(modifyFont:)
-			    keyEquivalent: @"s"];
+                            action: @selector(modifyFont:)
+                            keyEquivalent: @"s"];
       [menuItem setTag: NSSizeDownFontAction];
       [menuItem setTarget: self];
     }
@@ -974,5 +1013,171 @@ static Class		fontPanelClass = Nil;
     }
 }
 
-@end
+- (BOOL) addCollection: (NSString *)name options: (int)options
+{
+  [_collections setObject: [NSMutableArray arrayWithCapacity: 10] forKey: name];
+  return YES;
+}
 
+- (BOOL) removeCollection:(NSString *) collection;
+{
+  if ([_collections objectForKey: collection])
+    {
+      [_collections removeObjectForKey: collection];
+      return YES;
+    }
+  else
+    {
+      return NO;
+    }
+}
+
+- (NSArray *) collectionNames
+{
+  return [_collections allKeys];
+}
+
+- (void) addFontDescriptors: (NSArray *)descriptors 
+               toCollection: (NSString *)collection
+{
+  NSMutableArray *a = [_collections objectForKey: collection];
+
+  if (a)
+    {
+      [a addObjectsFromArray: descriptors];
+    }
+}
+
+- (void) removeFontDescriptor: (NSFontDescriptor *)descriptor 
+               fromCollection: (NSString *)collection
+{
+  NSMutableArray *a = [_collections objectForKey: collection];
+
+  if (a)
+    {
+      [a removeObject: descriptor];
+    }
+}
+- (NSArray *) fontDescriptorsInCollection: (NSString *)collection
+{
+  return [_collections objectForKey: collection];
+}
+
+- (NSArray *) availableFontNamesMatchingFontDescriptor: (NSFontDescriptor *)descriptor
+{
+  // FIXME
+  return nil;
+}
+
+- (NSDictionary *) convertAttributes: (NSDictionary *)attributes
+{
+  NSMutableDictionary *newAttributes;
+  unsigned int i;
+  float size;
+  float sizes[] = {4.0, 6.0, 8.0, 9.0, 10.0, 11.0, 12.0, 13.0, 
+                   14.0, 16.0, 18.0, 24.0, 36.0, 48.0, 64.0};
+  NSFontTraitMask t;
+
+  if (attributes == nil)
+    return nil;
+
+  newAttributes = AUTORELEASE([attributes mutableCopy]);
+  switch (_storedTag)
+    {
+      case NSNoFontChangeAction: 
+        break;
+      case NSViaPanelFontAction: 
+        // FIXME
+        break;
+      case NSAddTraitFontAction: 
+        t = [[attributes objectForKey: NSFontSymbolicTrait] unsignedIntValue];
+        
+        if (t & _trait)
+          {
+            return newAttributes;
+          }
+        else if (_trait == NSUnboldFontMask)
+          {
+            t &= ~NSBoldFontMask;
+          }
+        else if (_trait == NSUnitalicFontMask)
+          {
+            t &= ~NSItalicFontMask;
+          }
+        else
+          {
+            t &= _trait;
+            // FIXME: What about weight for NSBoldFontMask?
+          }
+        [newAttributes setObject: [NSNumber numberWithUnsignedInt: t]
+                       forKey: NSFontSymbolicTrait];
+        break;
+      case NSRemoveTraitFontAction: 
+        t = [[attributes objectForKey: NSFontSymbolicTrait] unsignedIntValue];
+        
+        if (!(t & _trait))
+          {
+            return newAttributes;
+          }
+        else if (_trait == NSUnboldFontMask)
+          {
+            t = (t | NSBoldFontMask) & ~NSUnboldFontMask;
+          }
+        else if (_trait == NSUnitalicFontMask)
+          {
+            t = (t | NSItalicFontMask) & ~NSUnitalicFontMask;
+          }
+        else
+          {
+            t &= ~_trait;
+            // FIXME: What about weight for NSBoldFontMask?
+          }
+        [newAttributes setObject: [NSNumber numberWithUnsignedInt: t]
+                       forKey: NSFontSymbolicTrait];
+        break;
+      case NSSizeUpFontAction: 
+        size = [[attributes objectForKey: NSFontSizeAttribute] floatValue];
+        for (i = 0; i < sizeof(sizes)/sizeof(float); i++)
+          {
+            if (sizes[i] > size)
+              {
+                size = sizes[i];
+                break;
+              }
+          }
+        [newAttributes setObject: [NSString stringWithFormat: @"%f", size]
+                       forKey: NSFontSizeAttribute];
+        break;
+      case NSSizeDownFontAction: 
+        size = [[attributes objectForKey: NSFontSizeAttribute] floatValue];
+        for (i = sizeof(sizes)/sizeof(float) -1; i >= 0; i--)
+          {
+            if (sizes[i] < size)
+              {
+                size = sizes[i];
+                break;
+              }
+          }
+        [newAttributes setObject: [NSString stringWithFormat: @"%f", size]
+                       forKey: NSFontSizeAttribute];
+        break;
+      case NSHeavierFontAction: 
+        // FIXME
+        break;
+      case NSLighterFontAction: 
+        // FIXME
+        break;
+    }
+
+  return newAttributes;
+}
+
+- (void) setSelectedAttributes: (NSDictionary *)attributes 
+                    isMultiple: (BOOL)flag
+{
+  ASSIGN(_selectedAttributes, attributes);
+  _multiple = flag;
+  DESTROY(_selectedFont);
+}
+
+@end
