@@ -12,25 +12,29 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */ 
 
 #include <Foundation/Foundation.h>
+#include <Foundation/NSKeyValueObserving.h>
+#include "AppKit/NSApplication.h"
+#include "AppKit/NSEvent.h"
+#include "AppKit/NSKeyValueBinding.h"
 #include "AppKit/NSPopUpButton.h"
 #include "AppKit/NSPopUpButtonCell.h"
-#include "AppKit/NSApplication.h"
 #include "AppKit/NSMenu.h"
 #include "AppKit/NSMenuItem.h"
 #include "AppKit/NSMenuView.h"
@@ -56,6 +60,8 @@ Class _nspopupbuttonCellClass = 0;
       // Initial version
       [self setVersion: 1];
       [self setCellClass: [NSPopUpButtonCell class]];
+
+      [self exposeBinding: NSSelectedIndexBinding];
     } 
 }
 
@@ -247,7 +253,9 @@ this to return nil to indicate that we have no context menu.
 
 - (void) selectItem: (id <NSMenuItem>)anObject
 {
+  [self willChangeValueForKey: NSSelectedIndexBinding];
   [_cell selectItem: anObject];
+  [self didChangeValueForKey: NSSelectedIndexBinding];
   [self synchronizeTitleAndSelectedItem];
 }
 
@@ -257,7 +265,9 @@ this to return nil to indicate that we have no context menu.
  */
 - (void) selectItemAtIndex: (int)index
 {
+  [self willChangeValueForKey: NSSelectedIndexBinding];
   [_cell selectItemAtIndex: index];
+  [self didChangeValueForKey: NSSelectedIndexBinding];
   [self synchronizeTitleAndSelectedItem];
 }
 
@@ -268,7 +278,9 @@ this to return nil to indicate that we have no context menu.
  */
 - (void) selectItemWithTitle: (NSString*)title
 {
+  [self willChangeValueForKey: NSSelectedIndexBinding];
   [_cell selectItemWithTitle: title];
+  [self didChangeValueForKey: NSSelectedIndexBinding];
   [self synchronizeTitleAndSelectedItem];
 }
 
@@ -515,4 +527,28 @@ this to return nil to indicate that we have no context menu.
   [super keyDown: theEvent];
 }
 
+- (void) setValue: (id)anObject forKey: (NSString*)aKey
+{
+  if ([aKey isEqual: NSSelectedIndexBinding])
+    {
+      [self selectItemAtIndex: [anObject intValue]];
+    }
+  else
+    {
+      [super setValue: anObject forKey: aKey];
+    }
+}
+
+- (id) valueForKey: (NSString*)aKey
+{
+  if ([aKey isEqual: NSSelectedIndexBinding])
+    {
+      return [NSNumber numberWithInt: [self indexOfSelectedItem]];
+    }
+  else
+    {
+      return [super valueForKey: aKey];
+    }
+}
+
 @end
