@@ -17,19 +17,20 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */ 
 
 #include <math.h>
@@ -62,21 +63,35 @@
 #define NSNUMBER(a) [NSNumber numberWithInt: (a)]
 #define NSFNUMBER(a) [NSNumber numberWithFloat: (a)]
 
+/*
+ * When a view gets printed it may need to be split up into segments, if the 
+ * views printed rectangle (after scaling) is bigger than the used area on the page.
+ * In this case we set xpages and ypages to the number of segments needed per 
+ * dimension. This pre-calculated value may not be accurate, as the view may 
+ * adjust the rect for each printed page.
+ * An independent concept is that multuple pages may be put on one sheet of paper.
+ * This is taken care of by nup and nupScale. Here we currently only allow even 
+ * values (or 1, when we don't use multiple pages). If we ever change this be 
+ * sure to change [NSView beginPageInRect:atPlacement:], perhaps by moving that 
+ * code to here?
+ * We always end up printing two rows per page, this is fine for 2, 4, 6, 8 and 10, 
+ * but starting from there it would be better to use three or four rows.
+ */
 /* Local pagination variables needed while printing */
 typedef struct _page_info_t {
   NSRect scaledBounds;       /* View's rect scaled by the user specified scale
-			        and page fitting */
+                                and page fitting */
   NSRect paperBounds;        /* Print area of a page in default user space, possibly
-				rotated if printing Landscape */
-  NSRect sheetBounds;        /* Print are of a page in default user space */
+                                rotated if printing Landscape */
+  NSRect sheetBounds;        /* Print area of a sheet in default user space */
   NSSize paperSize;          /* Size of the paper */
-  int xpages, ypages;
-  int first, last;
+  int xpages, ypages;        /* number of page segments for the view in both dimensions */
+  int first, last;           /* first and last page to print */
   double pageScale;          /* Scaling determined from page fitting */
   double printScale;         /* User specified scaling */
   double nupScale;           /* Scale required to fit nup pages on the sheet */
-  int    nup;                /* Number up pages to print on a sheet */
-  double lastWidth, lastHeight;
+  int    nup;                /* Number of pages to print on a sheet */
+  double lastWidth, lastHeight; /* max. values of last printed page (scaled) */
   NSPrintingOrientation orient;
   int    pageDirection;      /* NSPrintPageDirection */
 } page_info_t;
@@ -86,16 +101,16 @@ typedef struct _page_info_t {
 - (BOOL) _runOperation;
 - (void) _setupPrintInfo;
 - (void)_printOperationDidRun:(NSPrintOperation *)printOperation 
-		               returnCode:(int)returnCode
-		              contextInfo:(void *)contextInfo;
+                   returnCode:(int)returnCode
+                  contextInfo:(void *)contextInfo;
 - (void) _printPaginateWithInfo: (page_info_t *)info 
                      knowsRange: (BOOL)knowsRange;
 - (NSRect) _rectForPage: (int)page info: (page_info_t *)info 
-		              xpage: (int *)xptr
-		              ypage: (int *)yptr;
+                  xpage: (int *)xptr
+                  ypage: (int *)yptr;
 - (NSRect) _adjustPagesFirst: (int)first 
-			last: (int)last
-			info: (page_info_t *)info;
+                        last: (int)last
+                        info: (page_info_t *)info;
 - (void) _print;
 @end
 
@@ -106,12 +121,11 @@ typedef struct _page_info_t {
 
 @interface NSView (NSPrintOperation)
 - (void) _displayPageInRect: (NSRect)pageRect
-	        atPlacement: (NSPoint)location
-	           withInfo: (page_info_t)info;
-- (void) _endSheet;
+                   withInfo: (page_info_t)info;
 @end
 
 @interface NSView (NPrintOperationPrivate)
+- (void) _endSheet;
 - (void) _cleanupPrinting;
 @end
 
@@ -152,82 +166,82 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
 // Creating and Initializing an NSPrintOperation Object
 //
 + (NSPrintOperation *)EPSOperationWithView:(NSView *)aView
-				insideRect:(NSRect)rect
-				    toData:(NSMutableData *)data
+                                insideRect:(NSRect)rect
+                                    toData:(NSMutableData *)data
 {
-  return [self EPSOperationWithView: aView	
-	       insideRect: rect
-	       toData: data
-	       printInfo: nil];
+  return [self EPSOperationWithView: aView        
+               insideRect: rect
+               toData: data
+               printInfo: nil];
 }
 
-+ (NSPrintOperation *)EPSOperationWithView:(NSView *)aView	
-				insideRect:(NSRect)rect
-				    toData:(NSMutableData *)data
-				 printInfo:(NSPrintInfo *)aPrintInfo
++ (NSPrintOperation *)EPSOperationWithView:(NSView *)aView        
+                                insideRect:(NSRect)rect
+                                    toData:(NSMutableData *)data
+                                 printInfo:(NSPrintInfo *)aPrintInfo
 {
   return AUTORELEASE([[GSEPSPrintOperation alloc] initWithView: aView
-						  insideRect: rect
-						  toData: data
-						  printInfo: aPrintInfo]);
+                                                  insideRect: rect
+                                                  toData: data
+                                                  printInfo: aPrintInfo]);
 }
 
-+ (NSPrintOperation *)EPSOperationWithView:(NSView *)aView	
-				insideRect:(NSRect)rect
-				    toPath:(NSString *)path
-				 printInfo:(NSPrintInfo *)aPrintInfo
++ (NSPrintOperation *)EPSOperationWithView:(NSView *)aView        
+                                insideRect:(NSRect)rect
+                                    toPath:(NSString *)path
+                                 printInfo:(NSPrintInfo *)aPrintInfo
 {
-  return AUTORELEASE([[GSEPSPrintOperation alloc] initWithView: aView	
-						  insideRect: rect
-						  toPath: path
-						  printInfo: aPrintInfo]);
+  return AUTORELEASE([[GSEPSPrintOperation alloc] initWithView: aView        
+                                                  insideRect: rect
+                                                  toPath: path
+                                                  printInfo: aPrintInfo]);
 }
 
 + (NSPrintOperation *)printOperationWithView:(NSView *)aView
 {
   return [self printOperationWithView: aView
-	       printInfo: nil];
+               printInfo: nil];
 }
 
 + (NSPrintOperation *)printOperationWithView:(NSView *)aView
-				   printInfo:(NSPrintInfo *)aPrintInfo
+                                   printInfo:(NSPrintInfo *)aPrintInfo
 {
   return AUTORELEASE([[GSPrintOperation alloc] initWithView: aView
-					       printInfo: aPrintInfo]);
+                                               printInfo: aPrintInfo]);
 }
 
 + (NSPrintOperation *)PDFOperationWithView:(NSView *)aView 
-				insideRect:(NSRect)rect 
-				    toData:(NSMutableData *)data
+                                insideRect:(NSRect)rect 
+                                    toData:(NSMutableData *)data
 {
   return [self PDFOperationWithView: aView 
-	       insideRect: rect 
-	       toData: data 
-	       printInfo: nil];
+               insideRect: rect 
+               toData: data 
+               printInfo: nil];
 }
 
 + (NSPrintOperation *)PDFOperationWithView:(NSView *)aView 
-				insideRect:(NSRect)rect 
-				    toData:(NSMutableData *)data 
-				 printInfo:(NSPrintInfo*)aPrintInfo
+                                insideRect:(NSRect)rect 
+                                    toData:(NSMutableData *)data 
+                                 printInfo:(NSPrintInfo*)aPrintInfo
 {
   return AUTORELEASE([[GSPDFPrintOperation alloc] 
-			 initWithView: aView 
-			 insideRect: rect 
-			 toData: data 
-			 printInfo: aPrintInfo]);
+                         initWithView: aView 
+                         insideRect: rect 
+                         toData: data 
+                         printInfo: aPrintInfo]);
 }
 
 + (NSPrintOperation *)PDFOperationWithView:(NSView *)aView 
-				insideRect:(NSRect)rect 
-				    toPath:(NSString *)path 
-				 printInfo:(NSPrintInfo*)aPrintInfo
+                                insideRect:(NSRect)rect 
+                                    toPath:(NSString *)path 
+                                 printInfo:(NSPrintInfo*)aPrintInfo
 {
   return AUTORELEASE([[GSPDFPrintOperation alloc] 
-			 initWithView: aView 
-			 insideRect: rect 
-			 toPath: path 
-			 printInfo: aPrintInfo]);
+                         initWithView: aView 
+                         insideRect: rect 
+                         toPath: path 
+                         printInfo: aPrintInfo]);
 }
 
 //
@@ -267,25 +281,25 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
 //
 
 - (id)initEPSOperationWithView:(NSView *)aView
-		    insideRect:(NSRect)rect
-			toData:(NSMutableData *)data
-		     printInfo:(NSPrintInfo *)aPrintInfo
+                    insideRect:(NSRect)rect
+                        toData:(NSMutableData *)data
+                     printInfo:(NSPrintInfo *)aPrintInfo
 {
   RELEASE(self);
   
-  return [[GSEPSPrintOperation alloc] initWithView: aView	
-				      insideRect: rect
-				      toData: data
-				      printInfo: aPrintInfo];
+  return [[GSEPSPrintOperation alloc] initWithView: aView        
+                                      insideRect: rect
+                                      toData: data
+                                      printInfo: aPrintInfo];
 }
 
 - (id)initWithView:(NSView *)aView
-	 printInfo:(NSPrintInfo *)aPrintInfo
+         printInfo:(NSPrintInfo *)aPrintInfo
 {
   RELEASE(self);
   
   return [[GSPrintOperation alloc] initWithView: aView
-				   printInfo: aPrintInfo];
+                                   printInfo: aPrintInfo];
 }
 
 - (void) dealloc
@@ -490,10 +504,10 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
       [panel setAccessoryView: nil];
 
       if (button != NSOKButton)
-	{
-	  [self cleanUpOperation];
-	  return NO;
-	}
+        {
+          [self cleanUpOperation];
+          return NO;
+        }
       [panel finalWritePrintInfo];
     }
 
@@ -510,19 +524,22 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
 /** Run a print operation modally with respect to a window.
  */
 - (void)runOperationModalForWindow: (NSWindow *)docWindow 
-			  delegate: (id)delegate 
-		    didRunSelector: (SEL)didRunSelector 
-		       contextInfo:(void *)contextInfo
+                          delegate: (id)delegate 
+                    didRunSelector: (SEL)didRunSelector 
+                       contextInfo:(void *)contextInfo
 {
   NSMutableDictionary *dict;
   NSPrintPanel *panel = [self printPanel];
 
-  /* Save the selector so we can use it later */
-  dict = [_print_info dictionary];
-  [dict setObject: [NSValue value: &didRunSelector withObjCType: @encode(SEL)]
-	   forKey: @"GSModalRunSelector"];
-  [dict setObject: delegate
-	   forKey: @"GSModalRunDelegate"];
+  if (delegate != nil && didRunSelector != NULL)
+    {
+      /* Save the selector so we can use it later */
+      dict = [_print_info dictionary];
+      [dict setObject: [NSValue value: &didRunSelector withObjCType: @encode(SEL)]
+               forKey: @"GSModalRunSelector"];
+      [dict setObject: delegate
+               forKey: @"GSModalRunDelegate"];
+    }
 
   /* Assume we want to show the panel regardless of the value
      of _showPanels 
@@ -531,11 +548,11 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
   [self _setupPrintInfo];
   [panel updateFromPrintInfo];
   [panel beginSheetWithPrintInfo: _print_info 
-	          modalForWindow: docWindow 
-			delegate: self 
-		  didEndSelector: 
-		          @selector(_printOperationDidRun:returnCode:contextInfo:)
-		      contextInfo: contextInfo];
+                  modalForWindow: docWindow 
+                        delegate: self 
+                  didEndSelector: 
+                          @selector(_printOperationDidRun:returnCode:contextInfo:)
+                      contextInfo: contextInfo];
   [panel setAccessoryView: nil];
 }
 
@@ -595,14 +612,14 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
 @implementation NSPrintOperation (Private)
 
 - (id) initWithView:(NSView *)aView
-	 insideRect:(NSRect)rect
-	     toData:(NSMutableData *)data
-	  printInfo:(NSPrintInfo *)aPrintInfo
+         insideRect:(NSRect)rect
+             toData:(NSMutableData *)data
+          printInfo:(NSPrintInfo *)aPrintInfo
 {
   if ([NSPrintOperation currentOperation] != nil)
     {
       [NSException raise: NSPrintOperationExistsException
-		   format: @"There is already a printoperation for this thread"];
+                   format: @"There is already a printoperation for this thread"];
     }
 
   ASSIGN(_view, aView);
@@ -640,10 +657,10 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
   if (_page_order == NSUnknownPageOrder)
     {
       if ([[[_print_info dictionary] objectForKey: NSPrintReversePageOrder] 
-	    boolValue] == YES)
-	_page_order = NSDescendingPageOrder;
+            boolValue] == YES)
+        _page_order = NSDescendingPageOrder;
       else
-	_page_order = NSAscendingPageOrder;
+        _page_order = NSAscendingPageOrder;
     }
 
   [NSGraphicsContext setCurrentContext: _context];
@@ -658,7 +675,7 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
       [_view _cleanupPrinting];
       [NSGraphicsContext setCurrentContext: oldContext];
       NSRunAlertPanel(@"Error", @"Printing error: %@", 
-		      @"OK", NULL, NULL, localException);
+                      @"OK", NULL, NULL, localException);
     }
   NS_ENDHANDLER
   [self destroyContext];
@@ -683,8 +700,8 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
 }
 
 - (void)_printOperationDidRun:(NSPrintOperation *)printOperation 
-		   returnCode:(int)returnCode  
-		  contextInfo:(void *)contextInfo
+                   returnCode:(int)returnCode  
+                  contextInfo:(void *)contextInfo
 {
   id delegate;
   SEL didRunSelector;
@@ -697,21 +714,24 @@ static NSString *NSPrintOperationThreadKey = @"NSPrintOperationThreadKey";
       NSPrintPanel *panel = [self printPanel];
       [panel finalWritePrintInfo];
       if ([self _runOperation])
-	success = [self deliverResult];
+        success = [self deliverResult];
     }
   [self cleanUpOperation];
   dict = [_print_info dictionary];
-  [[dict objectForKey: @"GSModalRunSelector"] getValue:&didRunSelector];
+  [[dict objectForKey: @"GSModalRunSelector"] getValue: &didRunSelector];
   delegate = [dict objectForKey: @"GSModalRunDelegate"];
-  didRun = (void (*)(id, SEL, BOOL, id))[delegate methodForSelector: 
-						     didRunSelector];
-  didRun (delegate, didRunSelector, success, contextInfo);
+  if (delegate != nil && didRunSelector != NULL)
+    {
+      didRun = (void (*)(id, SEL, BOOL, id))[delegate methodForSelector: 
+                                                          didRunSelector];
+      didRun (delegate, didRunSelector, success, contextInfo);
+    }
 }
 
 
 
 
-
+/*
 static NSSize
 scaleSize(NSSize size, double scale)
 {
@@ -719,21 +739,23 @@ scaleSize(NSSize size, double scale)
   size.width  *= scale;
   return size;
 }
+*/
 
 static NSRect
 scaleRect(NSRect rect, double scale)
 {
   return NSMakeRect(NSMinX(rect) * scale,
-		    NSMinY(rect) * scale,
-		    NSWidth(rect) * scale,
-		    NSHeight(rect) * scale);
+                    NSMinY(rect) * scale,
+                    NSWidth(rect) * scale,
+                    NSHeight(rect) * scale);
 }
 
-/* Pagination - guess how many pages we need to print. This could be off
-   by one in both X and Y because of the view's ability to adjust the
-   width and height of the printRect during printing. Also set up a bunch
-   of other information needed for printing.
-*/
+/* 
+ * Pagination - guess how many pages we need to print. This could be off
+ * in both X and Y because of the view's ability to adjust the width and 
+ * height of the printRect during printing. 
+ * Also set up a bunch of other information needed for printing.
+ */
 - (void) _printPaginateWithInfo: (page_info_t *)info knowsRange: (BOOL)knowsRange
 {
   NSMutableDictionary *dict;
@@ -760,21 +782,25 @@ scaleRect(NSRect rect, double scale)
   info->paperBounds.size.height -= 
     ([_print_info topMargin]+[_print_info bottomMargin]);
 
-  info->sheetBounds = info->paperBounds;
   if (info->orient == NSLandscapeOrientation)
     {
       /* Bounding box needs to be in default user space, but the bbox
-	 we get is rotated */
+         we get is rotated */
       info->sheetBounds = NSMakeRect(NSMinY(info->paperBounds), 
-				     NSMinX(info->paperBounds), 
-				     NSHeight(info->paperBounds), 
-				     NSWidth(info->paperBounds));
+                                     NSMinX(info->paperBounds), 
+                                     NSHeight(info->paperBounds), 
+                                     NSWidth(info->paperBounds));
     }
+  else
+    {
+      info->sheetBounds = info->paperBounds;
+    }
+
   /* Save this for the view to look at */
   [dict setObject: [NSValue valueWithRect: info->paperBounds]
- 	   forKey: @"NSPrintPaperBounds"];
+            forKey: @"NSPrintPaperBounds"];
   [dict setObject: [NSValue valueWithRect: info->sheetBounds]
- 	   forKey: @"NSPrintSheetBounds"];
+            forKey: @"NSPrintSheetBounds"];
 
    /* Scale bounds by the user specified scaling */
   info->scaledBounds = scaleRect(_rect, info->printScale);
@@ -784,11 +810,11 @@ scaleRect(NSRect rect, double scale)
     {
       /* Now calculate page fitting to get page scale */
       if ([_print_info horizontalPagination] == NSFitPagination)
-	info->pageScale  = info->paperBounds.size.width 
-	  / NSWidth(info->scaledBounds);
+        info->pageScale  = info->paperBounds.size.width 
+          / NSWidth(info->scaledBounds);
       if ([_print_info verticalPagination] == NSFitPagination)
-	info->pageScale = MIN(info->pageScale,
-	  NSHeight(info->paperBounds)/NSHeight(info->scaledBounds));
+        info->pageScale = MIN(info->pageScale,
+          NSHeight(info->paperBounds)/NSHeight(info->scaledBounds));
       /* Scale bounds by pageScale */
       info->scaledBounds = scaleRect(info->scaledBounds, info->pageScale);
 
@@ -796,26 +822,27 @@ scaleRect(NSRect rect, double scale)
       info->xpages = ceil(NSWidth(info->scaledBounds)/NSWidth(info->paperBounds));
       info->ypages = ceil(NSHeight(info->scaledBounds)/NSHeight(info->paperBounds));
       if ([_print_info horizontalPagination] == NSClipPagination)
-	info->xpages = 1;
+        info->xpages = 1;
       if ([_print_info verticalPagination] == NSClipPagination)
-	info->ypages = 1;
+        info->ypages = 1;
     }
 
-  /* Calculate nup. If nup is an odd multiple of two, secretly change the
-     page orientation to it's complement to make pages fit better.
-  */
+  /*
+   * Calculate nup. If nup is an odd multiple of two, secretly change the
+   * page orientation to it's complement to make pages fit better.
+   */
   if (((int)(info->nup / 2) & 0x1) == 1)
     {
       float tmp;
       if (info->orient == NSLandscapeOrientation)
-	info->nupScale = 
-	  info->paperSize.width/(2*info->paperSize.height);
+        info->nupScale = 
+          info->paperSize.width/(2*info->paperSize.height);
       else
-	info->nupScale = 
-	  info->paperSize.height/(2*info->paperSize.width);
+        info->nupScale = 
+          info->paperSize.height/(2*info->paperSize.width);
       info->nupScale /= (info->nup / 2);
       info->orient = (info->orient == NSPortraitOrientation) ? 
-	NSLandscapeOrientation : NSPortraitOrientation;
+        NSLandscapeOrientation : NSPortraitOrientation;
       tmp = info->paperSize.width;
       info->paperSize.width = info->paperSize.height;
       info->paperSize.height = tmp;
@@ -837,8 +864,8 @@ scaleRect(NSRect rect, double scale)
    page. The returned pageRect is in the view's coordinate system
 */
 - (NSRect) _rectForPage: (int)page info: (page_info_t *)info 
-		  xpage: (int *)xptr
-		  ypage: (int *)yptr
+                  xpage: (int *)xptr
+                  ypage: (int *)yptr
 {
   int xpage, ypage;
   NSRect pageRect;
@@ -860,7 +887,7 @@ scaleRect(NSRect rect, double scale)
   if (ypage == 0)
     info->lastHeight = 0;
   pageRect = NSMakeRect(info->lastWidth, info->lastHeight,
-			NSWidth(info->paperBounds), NSHeight(info->paperBounds));
+                        NSWidth(info->paperBounds), NSHeight(info->paperBounds));
   pageRect = NSIntersectionRect(pageRect, info->scaledBounds);
   /* Scale to view's coordinate system */
   return scaleRect(pageRect, 1/(info->pageScale*info->printScale));
@@ -871,8 +898,8 @@ scaleRect(NSRect rect, double scale)
    _rectForPage:
 */
 - (NSRect) _adjustPagesFirst: (int)first 
-			last: (int)last 
-			info: (page_info_t *)info
+                        last: (int)last 
+                        info: (page_info_t *)info
 {
   int i, xpage, ypage;
   double hlimit, wlimit;
@@ -885,22 +912,22 @@ scaleRect(NSRect rect, double scale)
       pageRect = [self _rectForPage: i info: info xpage: &xpage ypage: &ypage];
       limitVal = NSMaxY(pageRect) - hlimit * NSHeight(pageRect);
       [_view adjustPageHeightNew: &newVal
-	                     top: NSMinY(pageRect)
-	                  bottom: NSMaxY(pageRect)
-	                   limit: limitVal];
+                             top: NSMinY(pageRect)
+                          bottom: NSMaxY(pageRect)
+                           limit: limitVal];
       if (newVal < NSMaxY(pageRect))
-	pageRect.size.height = MAX(newVal, limitVal) - NSMinY(pageRect);
+        pageRect.size.height = MAX(newVal, limitVal) - NSMinY(pageRect);
       limitVal = NSMaxX(pageRect) - wlimit * NSWidth(pageRect);
       [_view adjustPageWidthNew: &newVal
-	                   left: NSMinX(pageRect)
-	                  right: NSMaxX(pageRect)
-	                   limit: limitVal];
+                           left: NSMinX(pageRect)
+                          right: NSMaxX(pageRect)
+                           limit: limitVal];
       if (newVal < NSMaxX(pageRect))
-	pageRect.size.width = MAX(newVal, limitVal) - NSMinX(pageRect);
+        pageRect.size.width = MAX(newVal, limitVal) - NSMinX(pageRect);
       if (info->pageDirection == 0 || ypage == info->ypages - 1)
-	info->lastWidth = NSMaxX(pageRect)*(info->pageScale*info->printScale);
+        info->lastWidth = NSMaxX(pageRect)*(info->pageScale*info->printScale);
       if (info->pageDirection == 1 || xpage == info->xpages - 1)
-	info->lastHeight = NSMaxY(pageRect)*(info->pageScale*info->printScale);
+        info->lastHeight = NSMaxY(pageRect)*(info->pageScale*info->printScale);
     }
   return pageRect;
 }
@@ -916,7 +943,6 @@ scaleRect(NSRect rect, double scale)
   dict = [_print_info dictionary];
 
   /* Setup pagination */
-  allPages = [[dict objectForKey: NSPrintAllPages] boolValue];
   knowsPageRange = [_view knowsPageRange: &viewPageRange]; 
   [self _printPaginateWithInfo: &info knowsRange: knowsPageRange];
   if (knowsPageRange == NO)
@@ -925,12 +951,14 @@ scaleRect(NSRect rect, double scale)
     }
   else
     {
+      // These values never get used
       info.xpages = 1;
       info.ypages = viewPageRange.length;
     }
 
   [dict setObject: NSNUMBER(NSMaxRange(viewPageRange))
-	   forKey: @"NSPrintTotalPages"];
+           forKey: @"NSPrintTotalPages"];
+  allPages = [[dict objectForKey: NSPrintAllPages] boolValue];
   if (allPages == YES)
     {
       info.first = viewPageRange.location;
@@ -946,7 +974,6 @@ scaleRect(NSRect rect, double scale)
       info.last = MIN(info.last, (int)(NSMaxRange(viewPageRange) - 1));
       viewPageRange = NSMakeRange(info.first, (info.last-info.first)+1);
     }
-  info.lastWidth = info.lastHeight = 0;
   [dict setObject: NSFNUMBER(info.nupScale) forKey: @"NSNupScale"];
   [dict setObject: NSNUMBER(info.first) forKey: NSPrintFirstPage];
   if (allPages == YES && knowsPageRange == NO)
@@ -954,26 +981,36 @@ scaleRect(NSRect rect, double scale)
   else
     [dict setObject: NSNUMBER(info.last) forKey: NSPrintLastPage];
   NSDebugLLog(@"NSPrinting", @"Printing pages %d to %d", 
-	      info.first, info.last);
+              info.first, info.last);
   NSDebugLLog(@"NSPrinting", @"Printing rect %@, scaled %@",
-	      NSStringFromRect(_rect),
-	      NSStringFromRect(info.scaledBounds));
+              NSStringFromRect(_rect),
+              NSStringFromRect(info.scaledBounds));
 
-  _currentPage = info.first;
-  dir = 1;
   if (_page_order == NSDescendingPageOrder)
     {
       _currentPage = info.last;
       dir = -1;
     }
-  if (dir > 0 && _currentPage != 1)
+  else
+    {
+      _currentPage = info.first;
+      dir = 1;
+    }
+
+  /*
+   * FIXME: Independent of the page order we could pre-calculate the
+   * pageRects for all pages up to last (including clipping adjustment) 
+   * and use them when printing. 
+   */
+  info.lastWidth = info.lastHeight = 0;
+  if (!knowsPageRange && dir > 0 && _currentPage != 1)
     {
       /* Calculate page rects we aren't processing to catch up to the
-	 first page we are */
+         first page we are */
       NSRect pageRect;
       pageRect = [self _adjustPagesFirst: 1
-			           last: _currentPage-1 
-		                   info: &info];
+                                   last: _currentPage - 1 
+                                   info: &info];
     }
 
   /* Print the header information */
@@ -981,56 +1018,53 @@ scaleRect(NSRect rect, double scale)
 
   /* Print each page */
   i = 0;
-  while (i < (info.last-info.first+1))
+  while (i < (info.last - info.first + 1))
     {
-      NSPoint location;
-      NSRect pageRect, scaledPageRect;
+      NSRect pageRect;
+
       if (knowsPageRange == YES)
-	{
-	  pageRect = [_view rectForPage: _currentPage];
-	}
+        {
+          pageRect = [_view rectForPage: _currentPage];
+        }
       else
-	{
-	  if (dir < 0)
-	    pageRect = [self _adjustPagesFirst: 1 
-			                  last: _currentPage 
-			                  info: &info];
-	  else
-	    pageRect = [self _adjustPagesFirst: _currentPage 
-			                  last: _currentPage 
-			                  info: &info];
-	}
+        {
+          if (dir < 0)
+            pageRect = [self _adjustPagesFirst: 1 
+                                          last: _currentPage 
+                                          info: &info];
+          else
+            pageRect = [self _adjustPagesFirst: _currentPage 
+                                          last: _currentPage 
+                                          info: &info];
+        }
 
       NSDebugLLog(@"NSPrinting", @" current page %d, rect %@", 
-		  _currentPage, NSStringFromRect(pageRect));
+                  _currentPage, NSStringFromRect(pageRect));
       if (NSIsEmptyRect(pageRect))
-	break;
-
-      scaledPageRect = scaleRect(pageRect, info.printScale*info.pageScale);
-      location = [_view locationOfPrintRect: scaledPageRect];
+        break;
 
       /* Draw using our special view routine */
       [_view _displayPageInRect: pageRect
-	            atPlacement: location
-	               withInfo: info];
-	     
-      if (dir > 0 && _currentPage == info.last && allPages == YES)
-	{
-	  /* Check if adjust pages forced part of the bounds onto 
-	     another page */
-	  if (NSMaxX(pageRect) < NSMaxX(_rect) 
-	      && [_print_info horizontalPagination] != NSClipPagination)
-	    {
-	      info.xpages++;
-	    }
-	  if (NSMaxY(pageRect) < NSMaxY(_rect)
-	      && [_print_info verticalPagination] != NSClipPagination)
-	    {
-	      info.ypages++;
-	    }
-	  viewPageRange = NSMakeRange(1, (info.xpages * info.ypages));
-	  info.last = NSMaxRange(viewPageRange) - 1;
-	}
+                       withInfo: info];
+             
+      // We could end up in this case for each row/column not just the lase page.
+      if (!knowsPageRange && dir > 0 && _currentPage == info.last && allPages == YES)
+        {
+          /* Check if adjust pages forced part of the bounds onto 
+             another page */
+          if (NSMaxX(pageRect) < NSMaxX(_rect) 
+              && [_print_info horizontalPagination] != NSClipPagination)
+            {
+              info.xpages++;
+            }
+          if (NSMaxY(pageRect) < NSMaxY(_rect)
+              && [_print_info verticalPagination] != NSClipPagination)
+            {
+              info.ypages++;
+            }
+          viewPageRange = NSMakeRange(1, (info.xpages * info.ypages));
+          info.last = NSMaxRange(viewPageRange) - 1;
+        }
       i++;
       _currentPage += dir;
     } /* Print each page */
@@ -1038,7 +1072,7 @@ scaleRect(NSRect rect, double scale)
   /* Make sure we end the sheet */
   if (info.nup > 1 && (info.last - info.first) % info.nup != info.nup - 1)
     {
-      [_view drawSheetBorderWithSize: info.paperBounds.size];
+      [_view drawSheetBorderWithSize: info.sheetBounds.size];
       [_view _endSheet];
     }
   [_view endDocument];
@@ -1048,7 +1082,7 @@ scaleRect(NSRect rect, double scale)
   if (((int)(info.nup / 2) & 0x1) == 1)
     {
       info.orient = (info.orient == NSPortraitOrientation) ? 
-	NSLandscapeOrientation : NSPortraitOrientation;
+        NSLandscapeOrientation : NSPortraitOrientation;
       [dict setObject: NSNUMBER(info.orient) forKey: NSPrintOrientation];
     }
 }
@@ -1056,55 +1090,73 @@ scaleRect(NSRect rect, double scale)
 @end
 
 @implementation NSView (NSPrintOperation)
+
 - (void) _displayPageInRect: (NSRect)pageRect
-	        atPlacement: (NSPoint)location
-	           withInfo: (page_info_t)info
+                   withInfo: (page_info_t)info
 {
   int currentPage;
+  int numberOnSheet;
   float xoffset, yoffset, scale;
-  NSString *label;
+  NSPoint location;
   NSPrintOperation *printOp = [NSPrintOperation currentOperation];
   NSGraphicsContext *ctxt = [printOp context];
 
   currentPage = [printOp currentPage];
-
-  label = nil;
-  if (info.nup == 1)
-    label = [NSString stringWithFormat: @"%d", currentPage];
+  numberOnSheet = (currentPage - info.first) % info.nup;
 
   /* Begin a sheet (i.e. a physical page in Postscript terms). If 
      nup > 1 then this occurs only once every nup pages */
-  if ((currentPage - info.first) % info.nup == 0)
+  if (numberOnSheet == 0)
     {
+      NSString *label;
+
+      label = nil;
+      if (info.nup == 1)
+        label = [NSString stringWithFormat: @"%d", currentPage];
+
       [self beginPage: floor((currentPage - info.first)/info.nup)+1
-	    label: label
-	    bBox: info.sheetBounds
-	    fonts: nil];
+            label: label
+            bBox: info.sheetBounds
+            fonts: nil];
       if (info.orient == NSLandscapeOrientation)
-	{
-	  DPSrotate(ctxt, 90);
-	  DPStranslate(ctxt, 0, -info.paperSize.height);
-	}
+        {
+          DPSrotate(ctxt, 90);
+          DPStranslate(ctxt, 0, -info.paperSize.height);
+        }
       /* Also offset by margins */
       DPStranslate(ctxt, NSMinX(info.paperBounds), NSMinY(info.paperBounds));
+
+      /* End page setup for multi page */
+      if (info.nup != 1)
+        {
+          [self addToPageSetup];
+          [self endPageSetup];
+        }
     }
 
+  scale = info.pageScale * info.printScale;
+  location = [self locationOfPrintRect: scaleRect(pageRect, scale)];
+
   /* Begin a logical page */
   [self beginPageInRect: pageRect atPlacement: location];
-  scale = info.pageScale * info.printScale;
   if (scale != 1.0)
     DPSscale(ctxt, scale, scale);
+
+  /* FIXME: Why is this needed? Shouldn't the flip be handled by the lockFocus method? */
   if ([self isFlipped])
     {
-      NSAffineTransformStruct	ats = { 1, 0, 0, -1, 0, NSHeight(_bounds) };
+      NSAffineTransformStruct ats = { 1, 0, 0, -1, 0, NSHeight(_bounds) };
       NSAffineTransform *matrix, *flip;
 
       flip = [NSAffineTransform new];
       matrix = [NSAffineTransform new];
-      [matrix prependTransform: _boundsMatrix];
+      if (_boundsMatrix != nil)
+        {
+          [matrix prependTransform: _boundsMatrix];
+        }
       /*
        * The flipping process must result in a coordinate system that
-       * exactly overlays the original.	 To do that, we must translate
+       * exactly overlays the original. To do that, we must translate
        * the origin by the height of the view.
        */
       [flip setTransformStruct: ats];
@@ -1121,35 +1173,29 @@ scaleRect(NSRect rect, double scale)
   xoffset = 0 - NSMinX(pageRect);
   DPStranslate(ctxt, xoffset, yoffset);
 
-  if ((currentPage - info.first) % info.nup == 0)
-    [self endPageSetup];
+  /* End page setup for single page */
+  if (info.nup == 1)
+    {
+      [self addToPageSetup];
+      [self endPageSetup];
+    }
 
   /* Do the actual drawing */
-  [self displayRectIgnoringOpacity: pageRect];
+  [self displayRectIgnoringOpacity: pageRect inContext: ctxt];
 
   /* End a logical page */
-  DPSgrestore(ctxt); // Balance gsave in beginPageInRect:
-  [self drawPageBorderWithSize: 
-	   scaleSize(info.paperBounds.size, info.nupScale)];
+  // FIXME: Attempt to get the coordinates of the page border correct.
+  DPSgrestore(ctxt);
+  DPSgsave(ctxt);
+  [self drawPageBorderWithSize: info.paperBounds.size];
   [self endPage];
 
   /* End a physical page */
-  if (((currentPage - info.first) % info.nup == info.nup-1))
+  if (numberOnSheet == info.nup - 1)
     {
-      [self drawSheetBorderWithSize: info.paperBounds.size];
+      [self drawSheetBorderWithSize: info.sheetBounds.size];
       [self _endSheet];
     }
 }
 
-- (void) _endSheet
-{
-  NSPrintOperation *printOp = [NSPrintOperation currentOperation];
-  NSGraphicsContext *ctxt = [printOp context];
-
-  if ([printOp isEPSOperation] == NO)
-    DPSPrintf(ctxt, "showpage\n");
-  DPSPrintf(ctxt, "%%%%PageTrailer\n");
-  DPSPrintf(ctxt, "\n");
-}
-
 @end
