@@ -13,18 +13,20 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; see the file COPYING.LIB.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */
 
 #include <math.h>
@@ -33,14 +35,15 @@
 #include <Foundation/NSRunLoop.h>
 #include <Foundation/NSDebug.h>
 
+#include "AppKit/NSApplication.h"
+#include "AppKit/NSButtonCell.h"
+#include "AppKit/NSColor.h"
+#include "AppKit/NSEvent.h"
+#include "AppKit/NSImage.h"
+#include "AppKit/NSGraphics.h"
 #include "AppKit/NSScroller.h"
 #include "AppKit/NSScrollView.h"
 #include "AppKit/NSWindow.h"
-#include "AppKit/NSButtonCell.h"
-#include "AppKit/NSApplication.h"
-#include "AppKit/NSImage.h"
-#include "AppKit/NSColor.h"
-#include "AppKit/NSGraphics.h"
 
 /**<p>TODO Description</p>
  */
