@@ -2,7 +2,7 @@
 
    <abstract>Allows multiple views to share a region in a window</abstract>
 
-   Copyright (C) 1996, 1998, 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
+   Copyright (C) 1996, 1998, 1999, 2000, 2001, 2004, 2008 Free Software Foundation, Inc.
 
    Author: Robert Vasvari <vrobi@ddrummer.com>
    Date: Jul 1998
@@ -10,36 +10,38 @@
    Date: November 1998
    Author: Richard Frith-Macdonald <richard@brainstorm.co.uk>
    Date: January 1999
-   Author: Nicola Pero <n.pero@mi.flashnet.it>
-   Date: 2000, 2001
+   Author: Nicola Pero <nicola.pero@meta-innovation.com>
+   Date: 2000, 2001, 2008
 
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */
 
-#include <math.h>
 #include <Foundation/NSArray.h>
 #include <Foundation/NSCoder.h>
+#include <Foundation/NSDecimalNumber.h>
 #include <Foundation/NSDebug.h>
 #include <Foundation/NSException.h>
 #include <Foundation/NSGeometry.h>
 #include <Foundation/NSNotification.h>
 #include <Foundation/NSRunLoop.h>
 #include <Foundation/NSString.h>
+#include <Foundation/NSUserDefaults.h>
 
 #include "AppKit/NSApplication.h"
 #include "AppKit/NSColor.h"
@@ -83,6 +85,7 @@ static NSNotificationCenter *nc = nil;
   RELEASE(_backgroundColor);
   RELEASE(_dividerColor);
   RELEASE(_dimpleImage);
+  TEST_RELEASE(_autosaveName);
 
   if (_delegate != nil)
     {
@@ -93,6 +96,68 @@ static NSNotificationCenter *nc = nil;
   [super dealloc];
 }
 
+- (void) _autosaveSubviewProportions
+{
+  if (_autosaveName != nil) 
+    {
+      NSUserDefaults      *defaults;
+      NSString            *splitViewKey;
+      NSMutableDictionary *config;
+      NSArray	*subs = [self subviews];
+      unsigned	count = [subs count];
+      NSView	*views[count];
+
+      defaults  = [NSUserDefaults standardUserDefaults];
+      splitViewKey = [NSString stringWithFormat: @"NSSplitView Dividers %@", 
+			       _autosaveName];
+      config = [NSMutableDictionary new];
+
+      [subs getObjects: views];
+
+      /* Compute the proportions and store them.  */
+      if (count > 0)
+	{
+	  int i;
+	  float  oldTotal = 0.0;
+	  NSRect frames[count];
+	  
+	  for (i = 0; i < count; i++)
+	    {
+	      frames[i] = [views[i] frame];
+	      if (_isVertical == NO)
+		{
+		  oldTotal +=  NSHeight(frames[i]);
+		}
+	      else
+		{
+		  oldTotal +=  NSWidth(frames[i]);
+		}
+	    }
+
+	  for (i = 0; i < (count - 1); i++)
+	    {
+	      double proportion;
+	      
+	      if (_isVertical == NO)
+		{
+		  proportion = (NSHeight(frames[i]))/oldTotal;
+		}
+	      else
+		{
+		  proportion = (NSWidth(frames[i]))/oldTotal;	      
+		}
+	      
+	      [config setObject: [NSNumber numberWithDouble: proportion]
+		      forKey: [NSNumber numberWithInt: i]];
+	    }
+	}
+      
+      [defaults setObject: config  forKey: splitViewKey];
+      [defaults synchronize];
+      RELEASE (config);
+    }
+}
+
 - (BOOL) acceptsFirstMouse: (NSEvent*)theEvent
 {
   return YES;
@@ -603,6 +668,7 @@ static NSNotificationCenter *nc = nil;
   [nc postNotificationName: NSSplitViewDidResizeSubviewsNotification
 		    object: self];
 
+  [self _autosaveSubviewProportions];
 
   [self setNeedsDisplay: YES];
 
@@ -625,48 +691,117 @@ static NSNotificationCenter *nc = nil;
 
 - (void) adjustSubviews
 {
+  /* Try to load the autosaved view proportions, if any; if there are
+   * no autosaved view proportions, then manually adjust subviews
+   * proportionally to their current sizes.
+   */
   NSArray	*subs = [self subviews];
   unsigned	count = [subs count];
   NSView	*views[count];
-  NSRect	frames[count];
   NSSize	newSize;
   NSPoint	newPoint;
   unsigned	i;
-  NSRect	r;
-  float	oldTotal;
-  float	newTotal;
-  float	scale;
-  float	running;
-  
+  BOOL   autoloading = NO;
+  double proportions[count];
+
+  if (count == 0)
+    return;
+
   [nc postNotificationName: NSSplitViewWillResizeSubviewsNotification
       object: self];
-  
-  
+
+  /* Try loading the autosaved view proportions.  We store the
+   * proportions between the size of each view and the size of the
+   * whole area occupied by the views.  */
+  if (_autosaveName != nil) 
+    { 
+      NSUserDefaults     *defaults;
+      NSDictionary       *config;
+      NSString           *splitViewKey;
+      
+      defaults  = [NSUserDefaults standardUserDefaults];
+      splitViewKey = [NSString stringWithFormat: @"NSSplitView Dividers %@", 
+			       _autosaveName];
+      config = [defaults objectForKey: splitViewKey];
+      if (config != nil) 
+	{
+	  autoloading = YES;
+	  /* Please note that we don't save the 'proportion' of the
+	   * last view; it will be set up to take exactly the
+	   * remaining available space.  */
+	  for (i = 0; i < (count - 1); i++)
+	    {
+	      NSNumber *proportion = [config objectForKey: [NSNumber numberWithInt: i]];
+
+	      if (proportion == nil)
+		{
+		  /* If any autosaved proportion is missing, do not do
+		   * any autoloading.
+		   */
+		  autoloading = NO;
+		  break;
+		}
+	      else
+		{
+		  proportions[i] = [proportion doubleValue];
+		}
+	    }
+	}
+    }
+
   [subs getObjects: views];
-  if (_isVertical == NO)
+
+  /* If we haven't managed to load any saved view proportions, compute
+   * default proportions using the current view sizes.  */
+  if (autoloading == NO)
     {
-      newTotal = NSHeight(_bounds) - _dividerWidth*(count - 1);
-      oldTotal = 0.0;
+      float  oldTotal = 0.0;
+      NSRect frames[count];
+      
       for (i = 0; i < count; i++)
 	{
 	  frames[i] = [views[i] frame];
-	  oldTotal +=  NSHeight(frames[i]);
+	  if (_isVertical == NO)
+	    {
+	      oldTotal +=  NSHeight(frames[i]);
+	    }
+	  else
+	    {
+	      oldTotal +=  NSWidth(frames[i]);
+	    }
 	}
-      scale = newTotal/oldTotal;
-      running = 0.0;
+
+      for (i = 0; i < (count - 1); i++)
+	{
+	  if (_isVertical == NO)
+	    {
+	      proportions[i] = (NSHeight(frames[i]))/oldTotal;
+	    }
+	  else
+	    {
+	      proportions[i] = (NSWidth(frames[i]))/oldTotal;	      
+	    }
+	}
+    }
+
+  if (_isVertical == NO)
+    {
+      float newTotal = NSHeight(_bounds) - _dividerWidth*(count - 1);
+      float running = 0.0;
+
       for (i = 0; i < count; i++)
 	{
 	  float	newHeight;
 	  
-	  r = [views[i] frame];
-	  newHeight = NSHeight(frames[i]) * scale;
-	  if (i == count - 1)
+	  if (i < (count - 1))
 	    {
-	      newHeight = floor(newHeight);
+	      newHeight = proportions[i] * newTotal;
 	    }
 	  else
 	    {
-	      newHeight = ceil(newHeight);
+	      /* Size the last view to take exactly the remaining
+	       * space.  */
+	      newHeight = NSHeight(_bounds) - running;
 	    }
 	  newSize = NSMakeSize(NSWidth(_bounds), newHeight);
 	  newPoint = NSMakePoint(0.0, running);
@@ -677,27 +812,20 @@ static NSNotificationCenter *nc = nil;
     }
   else
     {
-      newTotal = NSWidth(_bounds) - _dividerWidth*(count - 1);
-      oldTotal = 0.0;
-      for (i = 0; i < count; i++)
-	{
-	  oldTotal +=  NSWidth([views[i] frame]);
-	}
-      scale = newTotal/oldTotal;
-      running = 0.0;
+      float newTotal = NSWidth(_bounds) - _dividerWidth*(count - 1);
+      float running = 0.0;
+
       for (i = 0; i < count; i++)
 	{
 	  float	newWidth;
 	  
-	  r = [views[i] frame];
-	  newWidth = NSWidth(r) * scale;
-	  if (i == count - 1)
+	  if (i < (count - 1))
 	    {
-	      newWidth = floor(newWidth);
+	      newWidth = proportions[i] * newTotal;
 	    }
 	  else
 	    {
-	      newWidth = ceil(newWidth);
+	      newWidth = NSWidth(_bounds) - running;
 	    }
 	  newSize = NSMakeSize(newWidth, NSHeight(_bounds));
 	  newPoint = NSMakePoint(running, 0.0);
@@ -711,6 +839,13 @@ static NSNotificationCenter *nc = nil;
     
   [nc postNotificationName: NSSplitViewDidResizeSubviewsNotification
       object: self];
+
+  /* Do not autosave if we just autoloaded - there's no point as it
+   * would be the same information we just loaded. ;-)  */
+  if (autoloading == NO)
+    {
+      [self _autosaveSubviewProportions];
+    }
 }
 
 - (float) dividerThickness 
@@ -776,13 +911,23 @@ static inline NSPoint centerSizeInRect(NSSize innerSize, NSRect outerRect)
 
 - (BOOL) isPaneSplitter
 {
-  // FIXME
+  /* TODO */
   return NO;
 }
 
 - (void) setIsPaneSplitter: (BOOL)flag
 {
-  // FIXME
+  /* TODO */
+}
+
+- (NSString *) autosaveName
+{
+  return _autosaveName;
+}
+
+- (void) setAutosaveName: (NSString *)autosaveName
+{
+  ASSIGN(_autosaveName, autosaveName);
 }
 
 /* Overridden Methods */
@@ -834,24 +979,8 @@ static inline NSPoint centerSizeInRect(NSSize innerSize, NSRect outerRect)
   [_window invalidateCursorRectsForView: self];
 }
 
-
-- (void) displayIfNeededInRectIgnoringOpacity: (NSRect)aRect
-{
-  if (_window == nil)
-    {
-      return;
-    }
-  
-  if (_never_displayed_before == YES)
-    {
-      _never_displayed_before = NO;
-      [self _adjustSubviews: _frame.size];
-    }
-
-  [super displayIfNeededInRectIgnoringOpacity: aRect];
-}
-
 - (void) displayRectIgnoringOpacity: (NSRect)aRect
+                          inContext: (NSGraphicsContext *)context
 {
   if (_window == nil)
     {
@@ -864,7 +993,7 @@ static inline NSPoint centerSizeInRect(NSSize innerSize, NSRect outerRect)
       [self _adjustSubviews: _frame.size];
     }
 
-  [super displayRectIgnoringOpacity: aRect];
+  [super displayRectIgnoringOpacity: aRect inContext: context];
 }
 
 - (id) delegate
@@ -897,24 +1026,31 @@ static inline NSPoint centerSizeInRect(NSSize innerSize, NSRect outerRect)
 - (void) encodeWithCoder: (NSCoder *)aCoder
 {
   [super encodeWithCoder: aCoder];
-
-  /*
-   *	Encode objects we don't own.
-   */
-  [aCoder encodeConditionalObject: _delegate];
-
-  /*
-   *	Encode the objects we do own.
-   */
-  [aCoder encodeObject: _dimpleImage];
-  [aCoder encodeObject: _backgroundColor];
-  [aCoder encodeObject: _dividerColor];
-
-  /*
-   *	Encode the rest of the ivar data.
-   */
-  [aCoder encodeValueOfObjCType: @encode(int) at: &_draggedBarWidth];
-  [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_isVertical];
+  if([aCoder allowsKeyedCoding])
+    {
+      [aCoder encodeBool: _isVertical forKey: @"NSIsVertical"];
+      [aCoder encodeObject: _autosaveName forKey: @"NSAutosaveName"];
+    }
+  else
+    {
+      /*
+       *	Encode objects we don't own.
+       */
+      [aCoder encodeConditionalObject: _delegate];
+      
+      /*
+       *	Encode the objects we do own.
+       */
+      [aCoder encodeObject: _dimpleImage];
+      [aCoder encodeObject: _backgroundColor];
+      [aCoder encodeObject: _dividerColor];
+      
+      /*
+       *	Encode the rest of the ivar data.
+       */
+      [aCoder encodeValueOfObjCType: @encode(int) at: &_draggedBarWidth];
+      [aCoder encodeValueOfObjCType: @encode(BOOL) at: &_isVertical];
+    }
 }
 
 - (id) initWithCoder: (NSCoder *)aDecoder
@@ -928,6 +1064,11 @@ static inline NSPoint centerSizeInRect(NSSize innerSize, NSRect outerRect)
 	  [self setVertical: [aDecoder decodeBoolForKey: @"NSIsVertical"]];
 	}
 
+      if ([aDecoder containsValueForKey: @"NSAutosaveName"])
+        {
+	  [self setAutosaveName: [aDecoder decodeObjectForKey: @"NSAutosaveName"]];
+	}
+
       _dividerWidth = [self dividerThickness];
       _draggedBarWidth = 8; // default bigger than dividerThickness
       ASSIGN(_dividerColor, [NSColor controlShadowColor]);
