@@ -8,19 +8,20 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */ 
 
 #include "AppKit/NSTableHeaderCell.h"
@@ -44,6 +45,7 @@
   [self setFont: [NSFont titleBarFontOfSize: 0]];
   // This is not exactly true 
   _cell.is_bezeled = YES;
+  _cell.is_bordered = NO;
 
   return self;
 }
@@ -58,12 +60,9 @@
   return NSInsetRect(theRect, borderSize.width, borderSize.height);
 }
 
-- (void) drawWithFrame: (NSRect)cellFrame
-		inView: (NSView *)controlView
+- (void) _drawBorderAndBackgroundWithFrame: (NSRect)cellFrame 
+                                    inView: (NSView*)controlView
 {
-  if (NSIsEmptyRect(cellFrame))
-    return;
-
   if (_cell.is_highlighted == YES)
     {
       [[GSTheme theme] drawButton: cellFrame withClip: cellFrame];
@@ -72,8 +71,6 @@
     {
       [[GSTheme theme] drawDarkButton: cellFrame withClip: cellFrame];
     }
-
-  [self drawInteriorWithFrame: cellFrame inView: controlView];
 }
 
 - (void) setHighlighted: (BOOL)flag
