@@ -14,19 +14,20 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */
 
 #include "config.h"
@@ -38,9 +39,13 @@
 
 #include "AppKit/NSApplication.h"
 #include "AppKit/NSCursor.h"
+#include "AppKit/NSGraphics.h"
 #include "AppKit/NSTextField.h"
 #include "AppKit/NSTextFieldCell.h"
 #include "AppKit/NSWindow.h"
+#include "AppKit/NSKeyValueBinding.h"
+#include "GSBindingHelpers.h"
+
 static NSNotificationCenter *nc;
 
 /*
@@ -61,6 +66,9 @@ static Class textFieldCellClass;
       textFieldCellClass = [NSTextFieldCell class];
       usedCellClass = textFieldCellClass;
       nc = [NSNotificationCenter defaultCenter];
+
+      [self exposeBinding: NSEditableBinding];
+      [self exposeBinding: NSTextColorBinding];
     }
 }
 
@@ -82,11 +90,12 @@ static Class textFieldCellClass;
 //
 - (id) initWithFrame: (NSRect)frameRect
 {
-  [super initWithFrame: frameRect];
-  [_cell setState: 1];
+  self = [super initWithFrame: frameRect];
+  if (self == nil)
+    return self;
+
   [_cell setBezeled: YES];
   [_cell setSelectable: YES];
-  [_cell setEnabled: YES];
   [_cell setEditable: YES];
   [_cell setDrawsBackground: YES];
   _text_object = nil;
@@ -98,13 +107,17 @@ static Class textFieldCellClass;
 {
   if (_delegate != nil)
     {
-      [nc removeObserver: _delegate  name: nil  object: self];
-      _delegate = nil;
+      [self setDelegate: nil];
     }
 
   [super dealloc];
 }
 
+- (BOOL) isFlipped
+{
+  return YES;
+}
+
 /** <p>Returns whether the NSTextField is editable. By default a NSTextField
     is not editable.</p>
     <p>See Also: -setEditable: [NSCell-isEditable]</p>
@@ -151,28 +164,28 @@ static Class textFieldCellClass;
   if ([self isSelectable] && (_super_view != nil))
     {
       if (_text_object)
-	[_text_object selectAll: self];
+        [_text_object selectAll: self];
       else
-	{
-	  NSText *text = [_window fieldEditor: YES  forObject: self];
-	  int length;
+        {
+          NSText *text = [_window fieldEditor: YES  forObject: self];
+          int length;
 
-	  if ([text superview] != nil)
-	    if ([text resignFirstResponder] == NO)
-	      return;
-	  
-	  //  [NSCursor hide];
-	  /* [self stringValue] generates a call to validateEditing 
-	     so we need to call it before setting up the _text_object */
-	  length = [[self stringValue] length];
-	  _text_object = [_cell setUpFieldEditorAttributes: text];
-	  [_cell selectWithFrame: _bounds
-		 inView: self
-		 editor: _text_object
-		 delegate: self
-		 start: 0
-		 length: length];
-	}
+          if ([text superview] != nil)
+            if ([text resignFirstResponder] == NO)
+              return;
+          
+          //  [NSCursor hide];
+          /* [self stringValue] generates a call to validateEditing 
+             so we need to call it before setting up the _text_object */
+          length = [[self stringValue] length];
+          _text_object = [_cell setUpFieldEditorAttributes: text];
+          [_cell selectWithFrame: _bounds
+                 inView: self
+                 editor: _text_object
+                 delegate: self
+                 start: 0
+                 length: length];
+        }
     }
 }
 
@@ -340,7 +353,10 @@ static Class textFieldCellClass;
 {
   if ([self isSelectable] == NO || [self isEnabled] == NO)
     {
-      [super mouseDown: theEvent];
+      if (_next_responder)
+        [_next_responder mouseDown: theEvent];
+      else
+        [super mouseDown: theEvent];
       return;
     }
 
@@ -371,22 +387,22 @@ static Class textFieldCellClass;
        * was in use by another control.
        */
       if ([_window makeFirstResponder: self])
-	{
-	  NSText *t = [_window fieldEditor: YES forObject: self];
-	  
-	  if ([t superview] != nil)
-	    {
-	      /* Can't take the field editor ... give up.  */
-	      return;
-	    }
-
-	  _text_object = [_cell setUpFieldEditorAttributes: t];
-	  [_cell editWithFrame: _bounds
-		 inView: self
-		 editor: _text_object
-		 delegate: self
-		 event: theEvent];
-	}
+        {
+          NSText *t = [_window fieldEditor: YES forObject: self];
+          
+          if ([t superview] != nil)
+            {
+              /* Can't take the field editor ... give up.  */
+                return;
+            }
+          
+          _text_object = [_cell setUpFieldEditorAttributes: t];
+          [_cell editWithFrame: _bounds
+                 inView: self
+                 editor: _text_object
+                 delegate: self
+                 event: theEvent];
+        }
     }
 }
 
@@ -452,39 +468,44 @@ static Class textFieldCellClass;
     {
       NSFormatter *formatter;
       NSString *string;
+      BOOL validatedOK = YES;
 
       formatter = [_cell formatter];
       string = AUTORELEASE ([[_text_object text] copy]);
 
-      if (formatter == nil)
-	{
-	  [_cell setStringValue: string];
-	}
-      else
-	{
-	  id newObjectValue;
-	  NSString *error;
- 
-	  if ([formatter getObjectValue: &newObjectValue 
-			 forString: string 
-			 errorDescription: &error] == YES)
-	    {
-	      [_cell setObjectValue: newObjectValue];
-	    }
-	  else
-	    {
-	      SEL sel = @selector(control:didFailToFormatString:errorDescription:);
+      if (formatter != nil)
+        {
+          id newObjectValue;
+          NSString *error;
+          
+          if ([formatter getObjectValue: &newObjectValue 
+                         forString: string 
+                         errorDescription: &error] == YES)
+            {
+              [_cell setObjectValue: newObjectValue];
+              return;
+            }
+          else
+            {
+              SEL sel = @selector(control:didFailToFormatString:errorDescription:);
+              
+              if ([_delegate respondsToSelector: sel])
+                {
+                  validatedOK = [_delegate control: self 
+                                           didFailToFormatString: string 
+                                           errorDescription: error];
+                }
+              else if (![string isEqualToString: @""])
+                {
+                  validatedOK = NO;
+                }
+            }
+        }
 
-	      if ([_delegate respondsToSelector: sel] && 
-		  [_delegate control: self 
-			     didFailToFormatString: string 
-			     errorDescription: error] == YES)
-		{
-		  [_cell setStringValue: string];
-		}
-	      
-	    }
-	}
+      if (validatedOK)
+        {
+          [_cell setStringValue: string];
+        }
     }
 }
 
@@ -515,35 +536,35 @@ static Class textFieldCellClass;
       
       partialString = [_text_object string];
       wasAccepted = [formatter isPartialStringValid: partialString 
-			       newEditingString: &newString 
-			       errorDescription: &error];
+                               newEditingString: &newString 
+                               errorDescription: &error];
 
       if (wasAccepted == NO)
-	{
-	  SEL sel = @selector(control:didFailToValidatePartialString:errorDescription:);
-	  
-	  if ([_delegate respondsToSelector: sel])
-	    {
-	      [_delegate control: self 
-			 didFailToValidatePartialString: partialString 
-			 errorDescription: error];
-	    }
-	}
+        {
+          SEL sel = @selector(control:didFailToValidatePartialString:errorDescription:);
+          
+          if ([_delegate respondsToSelector: sel])
+            {
+              [_delegate control: self 
+                         didFailToValidatePartialString: partialString 
+                         errorDescription: error];
+            }
+        }
 
       if (newString != nil)
-	{
-	  NSLog (@"Unimplemented: should set string to %@", newString);
-	  // FIXME ! This would reset editing !
-	  //[_text_object setString: newString];
-	}
+        {
+          NSLog (@"Unimplemented: should set string to %@", newString);
+          // FIXME ! This would reset editing !
+          //[_text_object setString: newString];
+        }
       else
-	{
-	  if (wasAccepted == NO)
-	    {
-	      // FIXME: Need to delete last typed character (?!)
-	      NSLog (@"Unimplemented: should delete last typed character");
-	    }
-	}
+        {
+          if (wasAccepted == NO)
+            {
+              // FIXME: Need to delete last typed character (?!)
+              NSLog (@"Unimplemented: should delete last typed character");
+            }
+        }
     }
 }
 
@@ -557,27 +578,27 @@ static Class textFieldCellClass;
   if (textMovement)
     {
       switch ([(NSNumber *)textMovement intValue])
-	{
-	case NSReturnTextMovement:
-	  if ([self sendAction: [self action] to: [self target]] == NO)
-	    {
-	      if ([self performKeyEquivalent: [_window currentEvent]] == NO)
-		[self selectText: self];
-	    }
-	  break;
-	case NSTabTextMovement:
-	  [_window selectKeyViewFollowingView: self];
+        {
+        case NSReturnTextMovement:
+          if ([self sendAction: [self action] to: [self target]] == NO)
+            {
+              if ([self performKeyEquivalent: [_window currentEvent]] == NO)
+                [self selectText: self];
+            }
+          break;
+        case NSTabTextMovement:
+          [_window selectKeyViewFollowingView: self];
 
-	  if ([_window firstResponder] == _window)
-	    [self selectText: self];
-	  break;
-	case NSBacktabTextMovement:
-	  [_window selectKeyViewPrecedingView: self];
+          if ([_window firstResponder] == _window)
+            [self selectText: self];
+          break;
+        case NSBacktabTextMovement:
+          [_window selectKeyViewPrecedingView: self];
 
-	  if ([_window firstResponder] == _window)
-	    [self selectText: self];
-	  break;
-	}
+          if ([_window firstResponder] == _window)
+            [self selectText: self];
+          break;
+        }
     }
 }
 
@@ -587,9 +608,9 @@ static Class textFieldCellClass;
     return NO;
   
   if (_delegate && [_delegate respondsToSelector: 
-				@selector(control:textShouldBeginEditing:)])
+                                @selector(control:textShouldBeginEditing:)])
     return [_delegate control: self 
-		      textShouldBeginEditing: textObject];
+                      textShouldBeginEditing: textObject];
   else 
     return YES;
 }
@@ -603,13 +624,13 @@ static Class textFieldCellClass;
     }
   
   if ([_delegate respondsToSelector: 
-		   @selector(control:textShouldEndEditing:)])
+                   @selector(control:textShouldEndEditing:)])
     {
       if ([_delegate control: self textShouldEndEditing: textObject] == NO)
-	{
-	  NSBeep ();
-	  return NO;
-	}
+        {
+          NSBeep ();
+          return NO;
+        }
     }
 
   if ([_delegate respondsToSelector: @selector(control:isValidObject:)] == YES)
@@ -620,14 +641,14 @@ static Class textFieldCellClass;
       formatter = [_cell formatter];
       
       if ([formatter getObjectValue: &newObjectValue 
-			  forString: [_text_object text] 
-		   errorDescription: NULL] == YES)
-	{
-	  if ([_delegate control: self isValidObject: newObjectValue] == NO)
-	    {
-	      return NO;
-	    }
-	}
+                          forString: [_text_object text] 
+                   errorDescription: NULL] == YES)
+        {
+          if ([_delegate control: self isValidObject: newObjectValue] == NO)
+            {
+              return NO;
+            }
+        }
     }
 
   // In all other cases
@@ -640,8 +661,8 @@ static Class textFieldCellClass;
       [_delegate respondsToSelector: @selector(control:textView:doCommandBySelector:)])
     {
       return [_delegate control: self 
-			textView: textView 
-			doCommandBySelector: command];
+                        textView: textView 
+                        doCommandBySelector: command];
     }
 
    return NO;
@@ -676,6 +697,16 @@ static Class textFieldCellClass;
   [_cell setTitleWithMnemonic: aString];
 }
 
+- (void)setBezelStyle:(NSTextFieldBezelStyle)style
+{
+  [_cell setBezelStyle: style];
+}
+
+- (NSTextFieldBezelStyle)bezelStyle
+{
+  return [_cell bezelStyle];
+}
+
 //
 // NSCoding protocol
 //
@@ -706,5 +737,41 @@ static Class textFieldCellClass;
   return self;
 }
 
+//
+// Bindings
+//
+
+/*
+ * Bindings implemented:
+ * alignment, hidden, editable, enabled, font, toolTip
+ *
+ * Bindings left to implement:
+ * other font bindings, value, displayPatternValue1
+ */
+
+- (void) bind: (NSString *)binding
+     toObject: (id)anObject
+  withKeyPath: (NSString *)keyPath
+      options: (NSDictionary *)options
+{
+  if ([binding hasPrefix: NSEditableBinding])
+    {
+      [self unbind: binding];
+      [[GSKeyValueAndBinding alloc] initWithBinding: NSEditableBinding 
+                                    withName: binding 
+                                    toObject: anObject
+                                    withKeyPath: keyPath
+                                    options: options
+                                    fromObject: self];
+    }
+  else
+    {
+      [super bind: binding
+         toObject: anObject
+      withKeyPath: keyPath
+          options: options];
+    }
+}
+
 @end
 
