@@ -30,23 +30,25 @@
    This file is part of the GNUstep GUI Library.
 
    This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
+   modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
+   You should have received a copy of the GNU Lesser General Public
    License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 */
 
 #include <Foundation/NSNotification.h>
 #include <Foundation/NSValue.h>
+#include "AppKit/NSAttributedString.h"
 #include "AppKit/NSGraphics.h"
 #include "AppKit/NSLayoutManager.h"
 #include "AppKit/NSScrollView.h"
@@ -1434,4 +1436,20 @@ insertion point. (see also: miswart)
   [self setRulerVisible: !_tf.is_ruler_visible];
 }
 
+- (void) outline: (id)sender
+{
+  // FIXME
+}
+
+- (void) setBaseWritingDirection: (NSWritingDirection)direction
+                           range: (NSRange)range
+{
+  // FIXME
+}
+
+- (void) toggleBaseWritingDirection: (id)sender
+{
+  // FIXME
+}
+
 @end
