@@ -8,17 +8,22 @@
 
    This file is part of the GNUstep Project
 
-   This library is free software; you can redistribute it and/or
+   This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
+   as published by the Free Software Foundation; either version 3
    of the License, or (at your option) any later version.
     
-   You should have received a copy of the GNU General Public  
-   License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-   */
+   You should have received a copy of the GNU General Public  
+   License along with this library; see the file COPYING.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
+*/
 
 #include <Foundation/NSArray.h>
 #include <Foundation/NSData.h>
@@ -51,18 +56,16 @@ int
 main(int argc, char** argv, char **env_c)
 {
   CREATE_AUTORELEASE_POOL(pool);
-  NSEnumerator *argEnumerator = nil;
-  NSWorkspace *workspace = nil;
+  NSEnumerator  *argEnumerator = nil;
+  NSWorkspace   *workspace = nil;
   NSFileManager *fm = nil;
-  NSString *arg = nil;
-  NSString 
-    *editor = nil,
-    *terminal = nil;
-  NSString 
-    *application = nil, 
-    *filetoopen = nil,
-    *filetoprint = nil,
-    *nxhost = nil;
+  NSString      *arg = nil;
+  NSString      *editor = nil;
+  NSString      *terminal = nil;
+  NSString      *application = nil;
+  NSString      *filetoopen = nil;
+  NSString      *filetoprint = nil;
+  NSString      *nxhost = nil;
 
 #ifdef GS_PASS_ARGUMENTS
   [NSProcessInfo initializeWithArguments:argv count:argc environment:env_c];
@@ -85,7 +88,9 @@ main(int argc, char** argv, char **env_c)
 
   if (application)
     {
-      // Don't start the application itself but use it for file opening. 
+      /* Start the application now,
+       * later on we will use it for file opening. 
+       */
       [workspace launchApplication: application];
     }
   
@@ -111,12 +116,13 @@ main(int argc, char** argv, char **env_c)
     {
       NSFileHandle	*fh = [NSFileHandle fileHandleWithStandardInput];
       NSData		*data = [fh readDataToEndOfFile];
-      NSString		*tempFile = [NSTemporaryDirectory()
-	stringByAppendingPathComponent: @"openfiletmp"];
-      NSNumber		*processId = [NSNumber numberWithInt:
-	[[NSProcessInfo processInfo] processIdentifier]];
+      NSString		*tempFile;
+      int		processId;
 
-      tempFile = [tempFile stringByAppendingString: [processId stringValue]];
+      tempFile = NSTemporaryDirectory();
+      tempFile = [tempFile stringByAppendingPathComponent: @"openfiletmp"];
+      processId = [[NSProcessInfo processInfo] processIdentifier];
+      tempFile = [tempFile stringByAppendingFormat: @"%d", processId];
       tempFile = [tempFile stringByAppendingString: @".txt"];
       [data writeToFile: tempFile atomically: YES];
       [workspace openFile: tempFile withApplication: editor];      
@@ -127,9 +133,9 @@ main(int argc, char** argv, char **env_c)
     {
       NSString *ext = [arg pathExtension];
       
-      if ([arg isEqualToString: @"-a"] || 
-	  [arg isEqualToString: @"-o"] || 
-	  [arg isEqualToString: @"-NXHost"])
+      if ([arg isEqualToString: @"-a"]
+        || [arg isEqualToString: @"-o"]
+        || [arg isEqualToString: @"-NXHost"])
 	{
 	  // skip since this is handled above...
 	  arg = [argEnumerator nextObject];
