@@ -1,5 +1,5 @@
 /* 
-   set_show_servicaes.m
+   set_show_service.m
 
    GNUstep utility to enable or disable a service for the current user.
 
@@ -10,15 +10,21 @@
    
    This file is part of the GNUstep Project
 
-   This library is free software; you can redistribute it and/or
+   This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
+   as published by the Free Software Foundation; either version 3
    of the License, or (at your option) any later version.
     
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
    You should have received a copy of the GNU General Public  
-   License along with this library; see the file COPYING.LIB.
-   If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   License along with this library; see the file COPYING.
+   If not, see <http://www.gnu.org/licenses/> or write to the 
+   Free Software Foundation, 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02110-1301, USA.
 
 */ 
 
@@ -60,7 +66,7 @@ main(int argc, char** argv, char **env)
 	{
 	  printf(
 "set_show_service enables or disables the display of a specified service\n"
-"item.  It's should be in the form 'set_show_services --enable name' or \n"
+"item.  It's should be in the form 'set_show_service --enable name' or \n"
 "'set_show_service --disable name' where 'name' is a service name.\n");
 	  exit(EXIT_SUCCESS);
 	}
